Package: epiregulon.extra
Title: Companion package to epiregulon with additional plotting,
        differential and graph functions
Version: 1.6.0
Authors@R: 
  c(person(given = "Xiaosai", family = "Yao", role = c("aut", "cre"), email = "xiaosai.yao@gmail.com", comment = c(ORCID = "0000-0001-9729-0726")),
    person(given = "Tomasz", family = "Włodarczyk", role = c("aut"), email = "tomwlo@gmail.com", comment = c(ORCID = "0000-0003-1554-9699")),
    person(given = "Timothy", family = "Keyes", role=c("aut"), email="keyes.timothy@gmail.com"),
    person(given = "Shang-Yang", family = "Chen", role = "aut", email = "sychen9584@gmail.com"))
Description: Gene regulatory networks model the underlying gene regulation hierarchies that drive gene expression and observed phenotypes. Epiregulon infers TF activity in single cells by constructing a gene regulatory network (regulons). This is achieved through integration of scATAC-seq and scRNA-seq data and incorporation of public bulk TF ChIP-seq data. Links between regulatory elements and their target genes are established by computing correlations between chromatin accessibility and gene expressions.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: scran, ComplexHeatmap, Matrix, SummarizedExperiment,
        checkmate, circlize, clusterProfiler, ggplot2, ggraph, igraph,
        patchwork, reshape2, scales, scater
Depends: R (>= 4.4), SingleCellExperiment
Suggests: epiregulon, knitr, rmarkdown, parallel, BiocStyle, testthat
        (>= 3.0.0), msigdb, GSEABase, dorothea, scMultiome, S4Vectors,
        scuttle, vdiffr, ggrastr, ggrepel
VignetteBuilder: knitr
URL: https://github.com/xiaosaiyao/epiregulon.extra/
biocViews: GeneRegulation, Network, GeneExpression, Transcription,
        ChipOnChip, DifferentialExpression, GeneTarget, Normalization,
        GraphAndNetwork
Config/testthat/edition: 3
BugReports: https://github.com/xiaosaiyao/epiregulon.extra/issues
LazyData: true
git_url: https://git.bioconductor.org/packages/epiregulon.extra
git_branch: RELEASE_3_22
git_last_commit: d60eef8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:55:45 UTC; biocbuild
Author: Xiaosai Yao [aut, cre] (ORCID: <https://orcid.org/0000-0001-9729-0726>),
  Tomasz Włodarczyk [aut] (ORCID:
    <https://orcid.org/0000-0003-1554-9699>),
  Timothy Keyes [aut],
  Shang-Yang Chen [aut]
Maintainer: Xiaosai Yao <xiaosai.yao@gmail.com>

# Generated by roxygen2: do not edit by hand

export(addCentrality)
export(buildDiffGraph)
export(buildGraph)
export(calculateJaccardSimilarity)
export(enrichPlot)
export(findDifferentialActivity)
export(findPartners)
export(getSigGenes)
export(normalizeCentrality)
export(permuteGraph)
export(plotActivityDim)
export(plotActivityViolin)
export(plotBubble)
export(plotDiffNetwork)
export(plotEpiregulonNetwork)
export(plotGseaNetwork)
export(plotHeatmapActivity)
export(plotHeatmapRegulon)
export(rankTfs)
export(regulonEnrich)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,colData)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggraph,create_layout)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_label)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(ggraph,theme_graph)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,each_edge)
importFrom(igraph,edge_attr)
importFrom(igraph,edge_attr_names)
importFrom(igraph,ego)
importFrom(igraph,ends)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,incident_edges)
importFrom(igraph,intersection)
importFrom(igraph,layout_with_sugiyama)
importFrom(igraph,list.vertex.attributes)
importFrom(igraph,rewire)
importFrom(igraph,shortest_paths)
importFrom(igraph,similarity)
importFrom(igraph,strength)
importFrom(igraph,subcomponent)
importFrom(igraph,subgraph)
importFrom(igraph,vcount)
importFrom(igraph,vertex_attr)

# Description

Companion package to `epiregulon` with additional plotting, differential and graph functions

# Installation

```
if(!require(BiocManager)) install.packages("BiocManager")
BiocManager::install("epiregulon.extra")
```

#' Creating graphs and related operations
#'
#' @description
#' The function enable to create graph objects using as input regulon objects returned by
#' `pruneRegulon` or `addWeights`. Both weighted and unweighted graphs can be
#' created that can further be visualized using dedicated functions.
#'
#' @details
#' \code{buildGraph} function creates a directed graph based on the output of
#' the \code{getRegulon} function. Four modes are available: (1) `tg` in which
#' connections are made directly between transcription factor and target genes. Even if
#' the same tf-tg pair is connected in the original regulon object through many
#' regulatory elements then only one edge is created. In such a case, when weighted
#' graph is created, weights are summarized by the aggregating function (by default
#' the maximum absolute value with the sign of the original value). Similarly, aggregation
#' is made in the `re` mode leaving only unique transcription factor-regulatory element pairs.
#' In `tripartite` mode edges connect transcription factors with regulatory elements and
#' regulatory elements with target genes. The same weights are used for both edges
#' that correspond to the single row in the regulon data frame (tf-re and re-tg). Note
#' that the original regulon structure is not fully preserved because each row is now
#' represented by two edges which are independent from each other. Thus they can be
#' coupled with different edges connected to the same regulatory element building the
#' path from transcription factor to the target gene of another transcription factor
#' through the shared regulatory element.
#'
#' \code{buildDiffGraph} a graph difference by subtracting the edges of \code{graph_obj_2}
#' from those of the \code{graph_obj_1}. If \code{weighted} is set to \code{TRUE} then for each
#' ordered pair of vertices (nodes) the difference in number of edges between \code{graph_obj_1}
#' and \code{graph_obj_1} is calculated. The result is used to set the number of
#' corresponding edges in output graph. Note that unless \code{abs_diff} is set to
#' \code{TRUE} any non-positive difference will translate into lack of the edges
#' for a corresponding ordered pair of vertices in the output graph (equivalent
#' to 0 value in the respective position in adjacency matrix). In case of
#' weighted graphs, the weight of the output graph is calculated as a difference
#' of the corresponding weights between input graphs.
#'
#' \code{addCentrality} calculates degree centrality for each vertex using
#' \code{igraph::strength}.
#'
#' With \code{normalizeCentrality} function the normalized values of centrality
#' are calculated from the original ones divided by
#' \code{FUN}(total number of non-zero edges associated with each node).
#'
#' \code{rankTfs} assign ranks to transcription factors according to degree
#' centrality of their vertices
#'
#' @param regulon an object returned by the getRegulon or addWeights function.
#' @param mode a character specifying which type of graph will be built. In \code{'tg'} mode
#' a bipartite graph is built by connecting transcription factors directly to the target genes
#' and ignoring information about mediating regulatory elements; in \code{'pairs'} mode
#' transcription factors are connected to unique target gene-regulatory element pairs;
#' in \code{'tripartite'} mode the network is made up of three types of vertices (nodes):
#' transcription factors, regulatory elements and target genes; here the path from
#' target gene to regulatory element always contains a regulatory element; in
#' \code{'re'} mode data in the target genes is dropped and only connections are
#' between transcription factors and regulatory elements.
#' @param graph,graph_obj_1,graph_obj_2  an igraph object.
#' @param weights a character specifying which variable should be used to assign
#' weights to edges.
#' @param cluster a character specifying the name of the cluster column which should be used
#' to retrieve weight values from regulon object. Using this argument makes sense
#' only with combination with \code{weights} parameter when it points to the regulon column
#' that is a matrix.
#' @param aggregation_function a function used to aggregate weights of duplicated edges,
#'  which might appear due to the many transcription factor converging at the same regulatory
#'  element; starting from this point each transcription factor is supposed to have a separate
#'  connection to the target gene, perhaps the same one across several connections. In
#'  \code{tripartite} mode this might result in many edges in the same node pair, however weights might
#'  differ since they are inherited from different tf-re-tg triplets (rows) in the
#'  \code{regulon} object. Similarly, duplicated edges are generated by one
#'  transcription factor using a regulatory element multiple times to reach different
#'  target genes. In \code{tg} mode the edges became duplicated if one transcription
#'  factor reaches the same target genes through many regulatory elements.
#' @param weighted a logical indicating whether weighted graphs are used; in \code{tripartite}
#' mode tf-re-tg triplet is decomposed into two edges corresponding to tf-re and re-tg pairs,
#' and both edges inherit the same weight, which was originally assigned to the parent triplet.
#' @param abs_diff a logical indicating whether absolute difference in the number
#' edges or their weights will be calculated.
#' @param FUN a function used for normalization. The input to this
#' function is be the number of edges connected with each node (incident edges).
#' @param type_attr a character corresponding to the name of the vertex attribute
#' which indicate the type of vertex.
#' @param na_replace a logical indicating whether NA values for weights should be
#' replaced with zeros.
#' @param keep_original_names A logical indicating whether gene names should be used as
#' node names in the output graph. Note that this might lead to the duplicated node
#' names if the same gene is present in two layers (transcription factors and target genes).
#' @param filter_edges A numeric defining the cutoff weight used for filtering out edges
#' which have weights equal or greater than cutoff. The isolated vertices are removed then
#' from the graph. Defaults to NULL in which case no filtering is applied.
#' @return an igraph object.
#' \code{rankTfs} returns a data.frame with transcription factors sorted according
#' to the value of the \code{centrality} attribute.
#' @examples
#' # create an artificial getRegulon output
#' set.seed(1234)
#' tf_set <- apply(expand.grid(LETTERS[1:10], LETTERS[1:10]),1,  paste, collapse = '')
#' regulon <- DataFrame(tf = sample(tf_set, 5e3, replace = TRUE))
#' gene_set <- expand.grid(LETTERS[1:10], LETTERS[1:10], LETTERS[1:10])
#' gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
#' regulon$target <- sample(gene_set, 5e3, replace = TRUE)
#' regulon$idxATAC <- 1:5e3
#' regulon$corr <- runif(5e3)*0.5+0.5
#' regulon$weights <- matrix(runif(15000), nrow=5000, ncol=3)
#' colnames(regulon$weights) <- c('all','cluster1', 'cluster2')
#' graph_tripartite <- buildGraph(regulon, cluster='all', mode = 'tripartite')
#'
#' # build bipartite graph using regulatory element-target gene pairs
#' graph_pairs_1 <- buildGraph(regulon, cluster = 'cluster1', mode = 'pairs')
#' graph_pairs_2 <- buildGraph(regulon, cluster = 'cluster2', mode = 'pairs')
#' graph_diff <- buildDiffGraph(graph_pairs_1, graph_pairs_2)
#' graph_diff <- addCentrality(graph_diff)
#' graph_diff <- normalizeCentrality(graph_diff)
#' tf_ranking <- rankTfs(graph_diff)
#'
#' @importFrom igraph V E graph_from_data_frame vcount delete_edges delete_vertices degree graph_from_adjacency_matrix as_adjacency_matrix V<- incident_edges strength edge_attr_names list.vertex.attributes
#' @export
buildGraph <- function(regulon,
                       mode = c("tg", "tripartite", "re", "pairs"),
                       weights = "weights",
                       cluster = "all",
                       aggregation_function = function(x) x[which.max(abs(x))],
                       na_replace = TRUE,
                       keep_original_names = TRUE,
                       filter_edges = NULL) {

  if (!is.null(weights) && !weights %in%
      colnames(regulon))
    stop(sprintf("%s column should be present in the regulon or `weights` argument must be set to NULL",
                 weights))
  mode <- match.arg(mode)
  # give names to the peaks and target genes which will be easy to extract
  regulon$idxATAC <- paste0(as.character(regulon$idxATAC),
                            "_peak")
  regulon$target <- paste0(regulon$target,
                           "_target_gene")
  vertex_columns <- switch(mode,
                           re = c("tf", "idxATAC"),
                           pairs = c("tf", "idxATAC","target"),
                           tripartite = c("idxATAC","target"),
                           tg = c("tf","target"))

  graph_data <- regulon[, vertex_columns]

  if (!is.null(weights)) {
    if (is.matrix(regulon[,weights])) {
      stopifnot(cluster %in% colnames(regulon[,weights]))
      weights_df <- data.frame(regulon[, weights][, cluster])
    } else weights_df <- data.frame(regulon[,weights])
    if (any(is.na(weights_df)) & na_replace) {
      message("Replacement of na values for weights with 0")
      weights_df[[1]][is.na(weights_df[[1]])] <- 0
    }
    colnames(weights_df) <- weights
    graph_data <- cbind(graph_data,
                        weights_df)
  }
  message(sprintf("Building graph using %s as edge weights",
                  weights))
  if (mode == "tripartite") {
    # add tf-re data
    colnames(graph_data) <- c("from", "to", weights)
    graph_data_tf_re <- data.frame(from = regulon$tf,
                                   to = regulon$idxATAC)
    if (!is.null(weights)) {
      graph_data_tf_re[, weights] <- weights_df[[1]]
    }
    graph_data <- rbind(graph_data,
                        graph_data_tf_re)
    rm(graph_data_tf_re)
    vertex_columns <- c("from", "to")
  }

  if (mode == "pairs") {
    # create node names corresponding to re-tg pairs
    graph_data$target <- paste(graph_data$idxATAC,
                               graph_data$target, sep = "@")
    graph_data <- graph_data[,c("tf", "target", weights)]
    vertex_columns <- c("tf","target")
  }

  if (is.null(weights)) {
    # avoid duplicated edges in the case of unweighted graph
    graph_data <- unique(graph_data)
  } else {
    colnames(graph_data)[colnames(graph_data) ==
                           weights] <- "weight"
    graph_data <- aggregate_edges(graph_data,
                                  aggregation_function)
  }

  epiregulon_graph <- graph_from_data_frame(graph_data)
  if (mode == "tripartite") {
    layer_numb <- rep(1, vcount(epiregulon_graph))
    layer_numb[grepl("_target_gene$",
                     V(epiregulon_graph)$name)] <- 2
    layer_numb[grepl("_peak$",
                     V(epiregulon_graph)$name)] <- 3
    V(epiregulon_graph)$layer <- layer_numb
  }
  # set 'type' attribute for vertices required by bipartite graphs
  vertex_type <- rep("transcription factor",
                     vcount(epiregulon_graph))
  vertex_type[grepl("_peak$",
                    V(epiregulon_graph)$name)] <- "regulatory element"
  vertex_type[grepl("_target_gene$",
                    V(epiregulon_graph)$name)] <- "target gene"
  V(epiregulon_graph)$type <- vertex_type
  # transform character constants to numeric values for later use by graphics
  # functions
  V(epiregulon_graph)$type.num <- match(V(epiregulon_graph)$type,
                                        c("transcription factor",
                                          "peak", "target gene"))

  # restore original names
  if (keep_original_names) {
    V(epiregulon_graph)$name <- gsub("_target_gene$|_peak$",
                                     "", V(epiregulon_graph)$name)
  }
  if (!is.null(filter_edges) &&
      !is.null(weights)) {
    epiregulon_graph <- delete_edges(epiregulon_graph,
                                     E(epiregulon_graph)[E(epiregulon_graph)$weight <=
                                                           filter_edges])
    epiregulon_graph <- delete_vertices(epiregulon_graph,
                                        V(epiregulon_graph)[degree(epiregulon_graph) ==
                                                              0])
  }
  epiregulon_graph
}

#' @rdname buildGraph
#' @export
buildDiffGraph <- function(graph_obj_1, graph_obj_2, weighted = TRUE, abs_diff = TRUE) {
  checkmate::assertClass(graph_obj_1, "igraph")
  checkmate::assertClass(graph_obj_2, "igraph")
  if (!identical(V(graph_obj_1)$name, V(graph_obj_2)$name)) {
    stop("The nodes should be the same in both graphs")
  }
  transformation_function <- ifelse(abs_diff, abs, identity)
  if (weighted) {
    res <- graph_from_adjacency_matrix(transformation_function(as_adjacency_matrix(graph_obj_1, attr = "weight") -
                                                                 as_adjacency_matrix(graph_obj_2, attr = "weight")), weighted = TRUE)
    # remove zero-weight edges
    res <- delete_edges(res, E(res)[E(res)$weight == 0])
  } else {
    res <- graph_from_adjacency_matrix(abs(as_adjacency_matrix(graph_obj_1) - as_adjacency_matrix(graph_obj_2)),
                                       weighted = FALSE)
  }

  if (!identical(V(graph_obj_1)$type, V(graph_obj_2)$type)) {
    warning("Types of nodes differ between graphs. Only those from the first graph are used.")
  }
  V(res)$type <- V(graph_obj_1)$type
  V(res)$type.num <- V(graph_obj_1)$type.num

  # remove nodes with no edges
  edge_numbers <- vapply(incident_edges(res, V(res), mode = "all"), length, FUN.VALUE = numeric(1))
  res <- delete_vertices(res, V(res)[edge_numbers == 0])
  res
}

#' @rdname buildGraph
#' @export
addCentrality <- function(graph) {
  checkmate::assertClass(graph, "igraph")
  V(graph)$centrality <- strength(graph)
  graph
}

#' @rdname buildGraph
#' @export
normalizeCentrality <- function(graph, FUN = sqrt, weighted = TRUE) {
  checkmate::assertClass(graph, "igraph")
  if (!"centrality" %in% list.vertex.attributes(graph))
    stop("Vertices do not have 'centrality' attribute")
  if (!"weight" %in% edge_attr_names(graph) & weighted)
    stop("Set 'weight' attribute to edges or use with 'weighted = FALSE'")

  # calculate number of edges for each node
  edge_numbers <- vapply(incident_edges(graph, V(graph), mode = "all"),
                         length, FUN.VALUE = numeric(1))

  V(graph)$centrality <- V(graph)$centrality/FUN(edge_numbers)
  graph
}

#' @rdname buildGraph
#' @export
rankTfs <- function(graph, type_attr = "type") {
  checkmate::assertClass(graph, "igraph")
  rank_df <- data.frame(tf = V(graph)$name[order(V(graph)$centrality[vertex_attr(graph,type_attr) == "transcription factor"],
                                                 decreasing = TRUE)],
                        centrality = sort(V(graph)$centrality[vertex_attr(graph, type_attr) == "transcription factor"],
                                          decreasing = TRUE))
  rank_df$rank <- base::rank(-rank_df$centrality)
  rank_df
}

#' Plot a graph build based on \code{getRegulon} output
#'
#' This function takes an input an igraph object created by any of the following:
#' \code{buildGraph}, \code{addCentrality}, \code{igraph::strength}, \code{normalizeCentrality}.
#' It makes a force-directed layout plot to visualize it at a high level.
#'
#' @param graph an igraph object
#' @param layout a layout specification. Any values that are valid for
#' \link[ggraph]{ggraph} or \link[ggraph]{create_layout} will work. Defaults to
#' 'stress'. Consider also trying 'mds', 'nicely', and 'fr' while you experiment.
#' @param label_size an integer indicating how large the labels of highlighted
#' transcription factors should be
#' @param tfs_to_highlight a character vector specifying which TFs in the plot
#' should be highlighted. Defaults to NULL (no labels).
#' @param edge_alpha a numeric value between 0 and 1 indicating the level of
#' transparency to use for the edge links in the force-directed layout. Defaults
#' to 0.02.
#' @param point_size a numeric value indicating the size of nodes in the force-directed layout
#' @param point_border_size a numeric value indicating the size of point
#' borders for nodes in the force-directed layout
#' @param label_alpha a numeric value between 0 and 1 indicating the level of
#' transparency to use for the labels of highlighted nodes
#' @param label_nudge_x a numeric value indicating the shift of the labels
#' along the x axis that should be used in the force-directed layout
#' @param label_nudge_y A numeric value indicating the shift of the labels
#' along the y axis that should be used in the force-directed layout.
#' @param ... optional additional arguments to pass to \link[ggraph]{create_layout}
#' @return a ggraph object
#' @importFrom ggraph create_layout ggraph geom_edge_link geom_node_label
#' @importFrom ggplot2 aes theme_void labs
#' @author Timothy Keyes, Tomasz Wlodarczyk
#' @examples
#' # create an artificial getRegulon output
#' set.seed(1234)
#' tf_set <- apply(expand.grid(LETTERS[seq_len(5)], LETTERS[seq_len(5)]),1,  paste, collapse = '')
#' regulon <- data.frame(tf = sample(tf_set, 5e2, replace = TRUE))
#' gene_set <- expand.grid(LETTERS[seq_len(5)], LETTERS[seq_len(5)], LETTERS[seq_len(5)])
#' gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
#' regulon$target <- sample(gene_set, 5e2, replace = TRUE)
#' regulon$idxATAC <- seq_len(5e2)
#' regulon$corr <- runif(5e2)*0.5+0.5
#' regulon$weights <- runif(500)
#' #create igraph object
#' graph_tripartite <- buildGraph(regulon, mode = 'tripartite')
#' plotEpiregulonNetwork(graph_tripartite, tfs_to_highlight = sample(unique(tf_set),3),
#' edge_alpha = 0.2)
#' @export
plotEpiregulonNetwork <- function(graph,
                                  layout = "stress", label_size = 3,
                                  tfs_to_highlight = NULL, edge_alpha = 0.02,
                                  point_size = 1, point_border_size = 0.5,
                                  label_alpha = 0.8, label_nudge_x = 0.2,
                                  label_nudge_y = 0.2, ...) {
  checkmate::assertClass(graph,
                         "igraph")
  my_layout <- create_layout(graph,
                             layout = layout, ...)
  highlighted <- my_layout[my_layout$name %in%
                             tfs_to_highlight, ]
  my_plot <- ggraph(graph = my_layout) +
    geom_edge_link(alpha = edge_alpha) +
    geom_node_point(aes(fill = type),
                    shape = 21, size = point_size,
                    stroke = point_border_size) +
    geom_node_label(aes(label = name),
                    data = highlighted, alpha = label_alpha,
                    nudge_x = label_nudge_x,
                    nudge_y = label_nudge_y,
                    size = label_size) +
    geom_node_point(aes(fill = type),
                    shape = 21, data = highlighted,
                    size = 3, stroke = point_border_size) +
    theme_void() + labs(fill = NULL)

  return(my_plot)
}

#' Plot graph according to grouping factor
#'
#' Plot graph with separate weights for different levels of the grouping factor
#'
#' @param regulon an object returned by the getRegulon or addWeights function
#' @param cutoff a numeric used to select values of the variables passed in `clusters`
#' parameter. Values greater than `cutoff` are retained and used as
#' graph edge weights.
#' @param tf a character vector storing the names of transcription factors to be
#' included in the graph
#' @param weight a string indicating the name of the column in the regulon to be used as
#' the weight of the edges
#' @param clusters a character vector indicating the clusters to be plotted
#' @param layout a layout specification. Any values that are valid for
#' \link[ggraph]{ggraph} or \link[ggraph]{create_layout} will work.
#' @author Xiaosai Yao, Tomasz Wlodarczyk
#' @return a ggraph object
#' @examples
#' #' # create an artificial getRegulon output
#' set.seed(1234)
#' tf_set <- apply(expand.grid(LETTERS[1:10], LETTERS[1:10]),1,  paste, collapse = '')
#' regulon <- S4Vectors::DataFrame(tf = sample(tf_set, 5e3, replace = TRUE))
#' gene_set <- expand.grid(LETTERS[1:10], LETTERS[1:10], LETTERS[1:10])
#' gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
#' regulon$target <- sample(gene_set, 5e3, replace = TRUE)
#' regulon$idxATAC <- 1:5e3
#' regulon$weight <- cbind(data.frame(C1 = runif(5e3), C2 = runif(5e3),
#' C3 = runif(5e3)))
#' plotDiffNetwork(regulon, tf = unique(tf_set)[1:3],
#' clusters = c('C1', 'C2', 'C3'), cutoff = 0.2)
#' @export

plotDiffNetwork <- function(regulon, cutoff = 0.01, tf = NULL,
                            weight = "weight", clusters, layout = "stress") {
  regulon.tf <- list()
  for (cluster in clusters) {
    # apply cutoff
    idx <- which(regulon$tf %in% tf & regulon[, weight][,cluster] > cutoff)
    regulon_cluster <- regulon[idx, c("tf", "target", weight)]

    # rename colnames as weight to be consistent across all groups
    regulon_cluster$weight <- regulon_cluster[, weight][,cluster]

    # rename tf to be tf_cluster
    regulon_cluster[, "tf"] <- paste0(regulon_cluster[,"tf"], "_", cluster)
    regulon.tf[[cluster]] <- regulon_cluster
  }

  combined.regulon <- do.call("rbind", regulon.tf)

  combined.graph <- buildGraph(combined.regulon, mode = "tg",
                               weights = "weight")

  plotEpiregulonNetwork(combined.graph, layout = layout,
                        tfs_to_highlight = unique(combined.regulon$tf), label_nudge_x = 0.1,
                        label_nudge_y = 0.1)
}

aggregate_edges <- function(graph_data, FUN) {
  grouping_factors <- paste(setdiff(colnames(graph_data), "weight"),
                            collapse = "+")
  aggregation_formula <- eval(parse(text = paste0("weight", "~",
                                                  grouping_factors)))
  stats::aggregate(graph_data, aggregation_formula, FUN)
}

edge_weights <- seq(0,1,length.out=20)
edge_weights[5:12] <- rev(edge_weights[5:12])
edge_weights[9] <- 0
edge_weights[7] <- NA

regulon = data.frame(tf = rep(LETTERS[1:10], each = 2), idxATAC = rep(1:10, 2), target  = rep(LETTERS[10:14], 4), corr = edge_weights)
regulon$target[10:13] <- LETTERS[15]
# duplicated tf-re and tf-tg pairs for aggregation
regulon[regulon$tf == "B", "idxATAC"] <- 3
regulon[regulon$tf == "C", "idxATAC"] <- 5
regulon[regulon$tf == "I", "target"] <- "K"
regulon[regulon$tf == "J", "target"] <- "M"

test_graph <- buildGraph(regulon, mode = "tripartite", weights = "corr")

re_list <- list()
target_list <- list()

# determine which regulatory elements and target genes can be reached from each transcription factor
# note that graph will disassemble original tf-re-tg triplets into tf-re and re-tg pairs.
for(tf in sort(base::unique(regulon$tf))){
  re_list[[tf]] <- sort(base::unique(as.character(regulon[regulon$tf == tf, "idxATAC"])))
  target_list[[tf]] <- sort(base::unique(regulon[regulon$idxATAC %in% re_list[[tf]], "target"]))
}

re_list_graph <- list()
target_list_graph <- list()
tfs <- igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor"]$name
for(tf in sort(base::unique(tfs))){
  nearby_nodes <- igraph::ego(test_graph, nodes = igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor" & igraph::V(test_graph)$name == tf], order = 2, mode = "out")
  re_list_graph[[tf]] <- sort(as.character(base::unique(nearby_nodes[[1]][nearby_nodes[[1]]$type == "regulatory element"]$name)))
  target_list_graph[[tf]] <- sort(as.character(base::unique(nearby_nodes[[1]][nearby_nodes[[1]]$type == "target gene"]$name)))
}

test_that("buildGraph function correctly connects vertices in in the tripartite mode", {
  expect_identical(re_list_graph, re_list)
  expect_identical(target_list_graph, target_list)
})

# find regulatory elements associated with respective tfs and target genes associated with regulatory elements
re_list <- list()
target_list <- list()
regulon$idxATAC <- as.character(regulon$idxATAC)
for(tf in sort(base::unique(regulon$tf))){
  re_list[[tf]] <- regulon[regulon$tf == tf, c("idxATAC", "corr"),drop = FALSE]
  re_list[[tf]][is.na(re_list[[tf]][["corr"]]),"corr"] <- 0
  re_list[[tf]] <- split(re_list[[tf]],re_list[[tf]]$idxATAC) |>
    lapply(function(x) data.frame(idxATAC = x$idxATAC[1], corr = x$corr[which.max(abs(x$corr))])) |>
    {function(x) do.call(rbind, x)}()
  re_list[[tf]] <- re_list[[tf]][base::order(re_list[[tf]]$corr),,drop = FALSE]
  re_list[[tf]] <- re_list[[tf]][base::order(re_list[[tf]]$idxATAC),,drop = FALSE]
  rownames(re_list[[tf]]) <- NULL
}

for(idxATAC in sort(base::unique(regulon$idxATAC))){
  target_list[[idxATAC]]<- regulon[regulon$idxATAC == idxATAC, c("target", "corr"),drop = FALSE]
  target_list[[idxATAC]][is.na(target_list[[idxATAC]][,"corr"]),"corr"] <- 0
  # aggregate edges
  target_list[[idxATAC]] <- split(target_list[[idxATAC]],target_list[[idxATAC]]$target) |>
    lapply(function(x) data.frame(target = x$target[1], corr = x$corr[which.max(abs(x$corr))])) |>
    {function(x) do.call(rbind, x)}()
  target_list[[idxATAC]] <- target_list[[idxATAC]][base::order(target_list[[idxATAC]]$corr),,FALSE]
  target_list[[idxATAC]] <- target_list[[idxATAC]][base::order(target_list[[idxATAC]]$target),,FALSE]
  rownames(target_list[[idxATAC]]) <- NULL
}

test_graph <- buildGraph(regulon, mode = "tripartite", weights = "corr")

tfs <- igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor"]$name
re_list_graph <- list()
edges <- igraph::as_edgelist(test_graph)
for(tf in sort(base::unique(tfs))){
  # find neighbouring nodes of tf of interest
  nearby_nodes <- igraph::ego(test_graph, nodes = igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor" & igraph::V(test_graph)$name == tf],
                              order = 1, mode = "out", mindist = 1)[[1]]
  from_ind <- which(edges[,1]==tf)
  df <- data.frame()
  # get weights of the edges  the neighbouring nodes
  for(i in seq_along(nearby_nodes)){
    to_ind <- which(edges[,2] == nearby_nodes[i]$name)
    neighbour_weights = E(test_graph)[intersect(from_ind, to_ind)]$weight
    df <- rbind(df, data.frame("idxATAC" = nearby_nodes[i]$name, "corr" = neighbour_weights))
  }
  df <- df[base::order(df$corr),]
  re_list_graph[[tf]] <- df[base::order(df$idxATAC),]
  rownames(re_list_graph[[tf]]) <- NULL
}

reg_elems <- igraph::V(test_graph)[igraph::V(test_graph)$type == "regulatory element"]$name
target_list_graph <- list()
for(re in sort(base::unique(reg_elems))){
  # find neighbouring downstream nodes of re of interest
  nearby_nodes <- igraph::ego(test_graph, nodes = igraph::V(test_graph)[igraph::V(test_graph)$type == "regulatory element" & igraph::V(test_graph)$name == re],
                              order = 1, mode = "out", mindist = 1)[[1]]
  from_ind <- which(edges[,1]==re)
  df <- data.frame()
  # get weights of the edges  the neighbouring nodes
  for(i in seq_along(nearby_nodes)){
    to_ind <- which(edges[,2] == nearby_nodes[i]$name)
    neighbour_weights = E(test_graph)[intersect(from_ind, to_ind)]$weight
    df <- rbind(df, data.frame("target" = nearby_nodes[i]$name, "corr" = neighbour_weights))
  }
  df <- df[base::order(df$corr),]
  target_list_graph[[re]] <- df[base::order(df$target),]
  rownames(target_list_graph[[re]]) <- NULL
}

test_that("buildGraph function correctly assigns weights in the tripartite mode", {
  expect_identical(re_list_graph, re_list)
  expect_identical(target_list_graph, target_list)
})

regulon_na_rem <- regulon
regulon_na_rem[is.na(regulon_na_rem$corr),"corr"] <- 0

# create second data frame defining network structure
regulon_2 <- regulon

# change part of the weights to have non-zero edges after graphs subtraction
regulon_2[regulon_2$target %in% c("K", "O", "J", "L"), "corr"] <-
  seq(0,1, length.out = sum(regulon_2$target %in% c("K", "O", "J", "L")))

# for some node part of the edge should be 0 after graph subtraction
regulon_2[which(regulon_2$target=="L")[1], "corr"] <- regulon[which(regulon$target=="L")[1], "corr"]
regulon_2[which(regulon_2$target=="L")[1:2], "corr"] <- regulon[which(regulon$target=="O")[1:2], "corr"]

aggregate_table <- function(tab, vertices_cols){
  pairs <- paste0(tab[[vertices_cols[1]]], "_", tab[[vertices_cols[2]]])
  split(tab, pairs) |> lapply(function(x) data.frame(c1 = x[[vertices_cols[1]]][1],
                                                     c2 = x[[vertices_cols[2]]][1],
                                                     corr = x$corr[which.max(abs(x$corr))])) |>
    {function(x) do.call(rbind, x)}() -> tab
  colnames(tab) <- c(vertices_cols, "corr")
  tab
}

# aggregate separately tf-re and re-tg pairs and calculate weight difrrerences
tf_re_table_1 <- regulon_na_rem[,c("tf", "idxATAC", "corr")] |> aggregate_table(c("tf","idxATAC"))
tf_re_table_2 <- regulon_2[,c("tf", "idxATAC", "corr")] |> aggregate_table(c("tf","idxATAC"))
tf_re_diff_table <- tf_re_table_1[,c("tf", "idxATAC")]
tf_re_diff_table$corr <- abs(tf_re_table_1$corr - tf_re_table_2$corr)
tf_re_diff_table <- tf_re_diff_table[tf_re_diff_table$corr!=0,]

re_tg_table_1 <- regulon_na_rem[,c("idxATAC", "target", "corr")] |> aggregate_table(c("idxATAC", "target"))
re_tg_table_2 <- regulon_2[,c("idxATAC", "target", "corr")] |> aggregate_table(c("idxATAC", "target"))
re_tg_diff_table <- re_tg_table_1[,c("idxATAC", "target")]
re_tg_diff_table$corr <- abs(re_tg_table_1$corr - re_tg_table_2$corr)
re_tg_diff_table <- re_tg_diff_table[re_tg_diff_table$corr!=0,]
diff_table <- merge(tf_re_diff_table[,1:2], re_tg_diff_table[1:2])

diff_regulon <- data.frame(tf=regulon$tf, idxATAC = regulon$idxATAC, target = regulon$target, corr=abs(regulon_na_rem$corr - regulon_2$corr))
diff_regulon <- diff_regulon[diff_regulon$corr!=0,]

test_graph_1 <- buildGraph(regulon, mode = "tripartite", weights = "corr")
test_graph_2 <- buildGraph(regulon_2, mode = "tripartite", weights = "corr")
test_diff_graph <- buildDiffGraph(test_graph_1, test_graph_2)

# find re and tg associated with each transcription factor
re_list <- split(diff_table$idxATAC, diff_table$tf) |> lapply(unique) |> lapply(sort) |> lapply(as.character)
target_list <- split(diff_table$target, diff_table$tf) |> lapply(unique) |> lapply(sort)

# the same based on the graph
re_list_graph <- list()
target_list_graph <- list()
tfs <- igraph::V(test_diff_graph)[igraph::V(test_diff_graph)$type == "transcription factor"]$name
for(tf in sort(base::unique(tfs))){
  nearby_nodes <- igraph::ego(test_diff_graph, nodes = igraph::V(test_diff_graph)[igraph::V(test_diff_graph)$type == "transcription factor" & igraph::V(test_diff_graph)$name == tf], order = 2, mode = "out")
  re_list_graph[[tf]] <- sort(as.character(base::unique(nearby_nodes[[1]][nearby_nodes[[1]]$type == "regulatory element"]$name)))
  target_list_graph[[tf]] <- sort(as.character(base::unique(nearby_nodes[[1]][nearby_nodes[[1]]$type == "target gene"]$name)))
}

test_that("buildDiffGraph function correctly connects vertices in in the tripartite mode", {
  expect_identical(re_list_graph, re_list)
  expect_identical(target_list_graph, target_list)
})

# test builDiffGraph with respect to weight assignment
# find regulatory elements associated with respective tfs, and target genes associated with regulatory elements
re_list <- split(tf_re_diff_table, tf_re_diff_table$tf) |>
  lapply(function(x) x[order(x$corr),c("idxATAC", "corr")]) |>
  lapply(function(x) {x <- x[order(x$idxATAC),]; rownames(x) <- NULL; x})

target_list <- split(re_tg_diff_table, re_tg_diff_table$idxATAC) |>
  lapply(function(x) x[order(x$corr),c("target", "corr")]) |>
  lapply(function(x) {x <- x[order(x$target),]; rownames(x) <- NULL; x})

test_graph <- buildDiffGraph(test_graph_1, test_graph_2)

tfs <- igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor"]$name
re_list_graph <- list()
edges <- igraph::as_edgelist(test_graph)
for(tf in sort(base::unique(tfs))){
  # find neighbouring nodes of tf of interest
  nearby_nodes <- igraph::ego(test_graph, nodes = igraph::V(test_graph)[igraph::V(test_graph)$type == "transcription factor" & igraph::V(test_graph)$name == tf],
                              order = 1, mode = "out", mindist = 1)[[1]]
  from_ind <- which(edges[,1]==tf)
  df <- data.frame()
  # get weights of the edges  the neighbouring nodes
  for(i in seq_along(nearby_nodes)){
    to_ind <- which(edges[,2] == nearby_nodes[i]$name)
    neighbour_weights = E(test_graph)[intersect(from_ind, to_ind)]$weight
    df <- rbind(df, data.frame("idxATAC" = nearby_nodes[i]$name, "corr" = neighbour_weights))
  }
  df <- df[base::order(df$corr),]
  re_list_graph[[tf]] <- df[base::order(df$idxATAC),]
  rownames(re_list_graph[[tf]]) <- NULL
}

reg_elems <- igraph::V(test_graph)[igraph::V(test_graph)$type == "regulatory element"]$name
target_list_graph <- list()
for(re in sort(base::unique(reg_elems))){
  # find neighbouring downstream nodes of re of interest
  nearby_nodes <- igraph::ego(test_graph, nodes = igraph::V(test_graph)[igraph::V(test_graph)$type == "regulatory element" & igraph::V(test_graph)$name == re],
                              order = 1, mode = "out", mindist = 1)[[1]]
  from_ind <- which(edges[,1]==re)
  df <- data.frame()
  # get weights of the edges  the neighbouring nodes
  for(i in seq_along(nearby_nodes)){
    to_ind <- which(edges[,2] == nearby_nodes[i]$name)
    neighbour_weights = E(test_graph)[intersect(from_ind, to_ind)]$weight
    df <- rbind(df, data.frame("target" = nearby_nodes[i]$name, "corr" = neighbour_weights))
  }
  df <- df[base::order(df$corr),]
  target_list_graph[[re]] <- df[base::order(df$target),]
  rownames(target_list_graph[[re]]) <- NULL
}

test_that("buildDiffGraph function correctly assigns weights in the tripartite mode", {
  expect_identical(re_list_graph, re_list)
  expect_identical(target_list_graph, target_list)
})

test_that("addCentrality throws the error when input is not an igraph",
          {expect_error(addCentrality("graph"), "Assertion on 'graph' failed: Must inherit from class 'igraph', but has class 'character'")}
)

set.seed(4201)

regulon <- data.frame(tf = rep(LETTERS[1:7], times = rmultinom(1,200, prob = c(0.02, 0.05, 0.1, 0.15, 0.2, 0.25, 0.28))[,1]),
                      target = c(rep(c("F", "G"),3), paste0("gene_", 1:30)[sample(1:30,194, replace=TRUE)]),
                      weights = runif(200))

tfs <- unique(regulon$tf)
regulon_aggregated <- data.frame()
for(tf in tfs){
  for (tg in unique(regulon[regulon$tf==tf,"target"])){
    weights <- regulon[regulon$tf==tf & regulon$target==tg, "weights"]
    regulon_aggregated <- rbind(regulon_aggregated, data.frame(tf=tf, target = tg, weights = weights[which.max(abs(weights))]))
  }
}
centralities <- unlist(lapply(tfs, function(x) sum(regulon_aggregated[regulon_aggregated$tf == x, "weights"])))
normalized_centralities <- setNames(unlist(lapply(tfs, function(x) sum(regulon_aggregated[regulon_aggregated$tf == x, "weights"])/sqrt(length(regulon_aggregated[regulon_aggregated$tf == x, "weights"])))), tfs)
test_graph <- buildGraph(regulon_aggregated)
test_graph <- addCentrality(test_graph)
res <- data.frame(tf = tfs, centrality = centralities, rank = rank(-centralities))
res <- res[order(res$centrality, decreasing = TRUE),]
rownames(res) <- as.integer(res$rank)
test_that("rankTfs works correctly",
          {expect_equal(rankTfs(test_graph), res, tolerance = 1e-8)}
)

test_graph <- normalizeCentrality(test_graph)

test_that("normalizeCentrality works correctly",
          {expect_equal(V(test_graph)[V(test_graph)$type == "transcription factor"]$centrality, as.vector(normalized_centralities[V(test_graph)[V(test_graph)$type == "transcription factor"]$name]))}
)

#' @importFrom ggplot2 element_text scale_color_gradient
plotActivityDim_ <- function(sce, activity_matrix, tf, dimtype,
    label, legend.label, colors, limit, ...) {

    tf.activity <- as.numeric(activity_matrix[tf, ])
    sce$activity <- tf.activity

    g <- scater::plotReducedDim(sce, dimred = dimtype, colour_by = "activity",
        text_by = label, ...)

    g <- g + scale_color_gradient(low = colors[1], high = colors[2],
        limit = limit, oob = scales::squish) + ggtitle(tf) +
        labs(color = legend.label) + theme_classic(base_size = 12) +
        theme(plot.title = element_text(hjust = 0.5))

    return(g)

}

#' Plot cell-level reduced dimension results stored in a SingleCellExperiment object, colored by activities for a list of TFs
#'
#' @param sce  A SingleCellExperiment object containing dimensionality reduction coordinates
#' @param activity_matrix A matrix of TF activities inferred from calculateActivity
#' @param tf A character vector indicating the names of the transcription factors to be plotted
#' @param dimtype String indicating the name of dimensionality reduction matrix to be extracted from the SingleCellExperiment
#' @param label String corresponding to the field in the colData of sce for annotation on plot
#' @param ncol A integer to specify the number of columns in the combined plot, if combine == TRUE
#' @param nrow A integer to specify the number of rows in the combined plot, if combine == TRUE
#' @param title A string to specify the name of the combined plot
#' @param combine logical to indicate whether to combine and visualize the plots in one panel
#' @param legend.label String indicating the name of variable to be plotted on the legend
#' @param colors A vector of 2 colors for the intensity, with the first element referring to the lower value and
#' the second element referring to the higher value. Default is c('blue','yellow').
#' @param limit A vector of lower and upper bounds for the color scale. The default option is NULL and will adjust
#' to minimal and maximal values
#' @param ... Additional arguments from scater::plotReducedDim
#'
#' @return A combined ggplot object or a list of ggplots if combine == FALSE
#' @importFrom SummarizedExperiment colData
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' @export
#'
#' @examples
#' # create a mock singleCellExperiment object for gene expression matrix
#' example_sce <- scuttle::mockSCE()
#' example_sce <- scuttle::logNormCounts(example_sce)
#' example_sce <- scater::runPCA(example_sce)
#' example_sce <- scater::runUMAP(example_sce)
#' example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
#' plotActivityDim(sce = example_sce, activity = logcounts(example_sce),
#' tf = c('Gene_0001','Gene_0002'),  label = 'cluster')

#' @author Xiaosai Yao, Shang-yang Chen
#'
plotActivityDim <- function(sce = NULL,
    activity_matrix, tf, dimtype = "UMAP",
    label = NULL, ncol = NULL, nrow = NULL,
    title = NULL, combine = TRUE, legend.label = "activity",
    colors = c("blue", "yellow"), limit = NULL,
    ...) {

    # give warning for genes absent in tf list
    missing <- tf[which(!tf %in% rownames(activity_matrix))]
    if (!identical(missing, character(0))) {
        writeLines(paste0(missing,
            " not found in activity matrix. Excluded from plots"))
    }

    tf <- tf[which(tf %in% rownames(activity_matrix))]


    gs <- lapply(tf, function(x) {
        suppressMessages(return(plotActivityDim_(sce,
            activity_matrix, x, dimtype,
            label, legend.label, colors,
            limit, ...)))
    })

    if (combine == TRUE) {

        gs <- patchwork::wrap_plots(gs,
            ncol = ncol, nrow = nrow) +
            patchwork::plot_annotation(title = title)

        return(gs)

    } else {

        return(gs)

    }

}


#' @importFrom ggplot2 ggplot geom_violin theme_classic ggtitle ylab theme scale_fill_manual facet_grid geom_boxplot
plotActivityViolin_ <- function(activity_matrix, tf, clusters,
    legend.label, colors, text_size, facet_grid_variable,
    boxplot) {

    tf.activity <- as.numeric(activity_matrix[tf, ])

    df <- data.frame(activity = tf.activity, clusters = clusters)

    if (!is.null(facet_grid_variable)) {
        df$facet <- facet_grid_variable
    }


    g <- ggplot(df, aes(x = clusters,
        y = activity, fill = clusters)) + geom_violin() +
        theme_classic(base_size = 12) + ggtitle(tf) + ylab(legend.label) +
        theme(legend.position = "none", plot.title = element_text(hjust = 0.5),
            axis.text.x = element_text(angle = 90, hjust = 1))

    if (!is.null(colors)) {
        g <- g + scale_fill_manual(values = colors)
    }

    if (!is.null(facet_grid_variable)) {
        g <- g + facet_grid(stats::reformulate("facet",
            "."), scales = "free", space = "free")
    }

    if (boxplot) {
        g <- g + geom_boxplot(width = 0.1)
    }
    g <- g + theme(text = element_text(size = text_size))


    return(g)

}

#' Generate violin plots of inferred activities for a list of TFs grouped by cluster/group labels
#'
#' @param activity_matrix A matrix of TF activities inferred from calculateActivity
#' @param tf A character vector indicating the names of the transcription factors to be plotted
#' @param clusters A vector of cluster or group labels for single cells
#' @param ncol A integer to indicate the number of columns in the combined plot, if `combine = TRUE`
#' @param nrow A integer to indicate the number of rows in the combined plot, if `combine = TRUE`
#' @param combine logical to indicate whether to combine and visualize the plots in one panel
#' @param legend.label String indicating the name of variable to be plotted on the legend
#' @param colors  A character vector representing the names of colors
#' @param title String indicating the title of the plot if `combine = TRUE`
#' @param text_size Scalar indicating the font size of the title
#' @param facet_grid_variable  A character vector of a secondary label to split the plots by facet_grid
#' @param boxplot logical indicating whether to add boxplot on top of violin plot
#'
#' @return A combined ggplot object or a list of ggplots if `combine = FALSE`
#' @export
#'
#' @examples
#' # create a mock singleCellExperiment object for gene expression matrix
#' example_sce <- scuttle::mockSCE()
#' example_sce <- scuttle::logNormCounts(example_sce)
#' example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
#' plotActivityViolin(activity_matrix = logcounts(example_sce),
#' tf = c('Gene_0001','Gene_0002'),  clusters = example_sce$cluster)
#'
#' @author Xiaosai Yao, Shang-yang Chen
plotActivityViolin <- function(activity_matrix, tf, clusters,
    ncol = NULL, nrow = NULL, combine = TRUE, legend.label = "activity",
    colors = NULL, title = NULL, text_size = 10, facet_grid_variable = NULL,
    boxplot = FALSE) {

    # give warning for genes absent in tf list
    missing <- tf[which(!tf %in% rownames(activity_matrix))]
    if (!identical(missing, character(0))) {
        message(missing, " not found in activity matrix. Excluded from plots")
    }
    tf <- tf[which(tf %in% rownames(activity_matrix))]

    gs <- lapply(tf, function(x) {
        return(plotActivityViolin_(activity_matrix, x,
            clusters, legend.label, colors, text_size,
            facet_grid_variable, boxplot))
    })

    if (combine == TRUE) {

        gs <- patchwork::wrap_plots(gs, ncol = ncol, nrow = nrow) +
            patchwork::plot_annotation(title = title,
                theme = theme(plot.title = element_text(hjust = 0.5)))

        return(gs)

    } else {

        return(gs)

    }

}

#' Generate bubble plots of relative activities across cluster/group labels for a list of TFs
#'
#' @param activity_matrix A matrix of TF activities inferred from calculateActivity
#' @param tf A character vector indicating the names of the transcription factors to be plotted
#' @param clusters A character or integer vector of cluster or group labels for single cells
#' @param bubblesize String indicating the variable from findDifferentialActivity output to scale size of bubbles
#' by either `log.FDR`, `summary.logFC` or `summary.diff`. Default is `logFDR`.
#' @param color.theme String indicating the color theme used for the bubble plot and corresponding to the color options
#' in `scale_color_viridis_c`
#' @param legend.label String indicating the name of legend corresponding to the color scale
#' @param x.label String indicating the x axis label
#' @param y.label String indicating the y axis label
#' @param title String indicating the title of the plot
#' @param ... Additional arguments to pass to `findDifferentialActivity`
#'
#' @return A ggplot object
#' @export
#' @importFrom ggplot2 ggplot aes geom_point scale_color_viridis_c scale_size_continuous theme_classic theme labs ylab xlab ggtitle
#'
#' @examples
#' example_sce <- scuttle::mockSCE()
#' example_sce <- scuttle::logNormCounts(example_sce)
#' example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
#' plotBubble(activity_matrix = logcounts(example_sce),
#' tf = c('Gene_0001','Gene_0002'),  clusters = example_sce$cluster)
#' @author Shang-yang Chen
plotBubble <- function(activity_matrix, tf, clusters,
                       bubblesize = c("log.FDR", "summary.logFC","summary.diff"),
                       color.theme = "viridis",
                       legend.label = "relative_activity",
                       x.label = "clusters",
                       y.label = "transcription factors",
                       title = "TF activity",
                       ...) {

  bubblesize <- match.arg(bubblesize)

  # give warning for genes absent in tf list
  missing <- tf[which(!tf %in% rownames(activity_matrix))]
  if (!identical(missing, character(0))) {
    message(missing, " not found in activity matrix. Excluded from plots")
  }
  tf <- tf[which(tf %in% rownames(activity_matrix))]

  # find logFC and FDR of TFs
  markers <- findDifferentialActivity(activity_matrix, clusters, log.p = TRUE, ...)
  markers <- suppressMessages(getSigGenes(markers,
                                          fdr_cutoff = 1.5,
                                          summary_cutoff = -100))
  markers <- markers[which(markers$tf %in% tf),]
  levels <- make.names(unique(tf[tf %in% markers$tf]))
  markers$tf <- make.names(markers$tf)

  # rename markers
  colnames(markers)[colnames(markers) == "class"] <- "clusters"

  # z normalize activity and compute mean by cluster
  tf.activity <- activity_matrix[tf, , drop = FALSE]
  df <- data.frame(clusters = clusters, t(as.matrix(tf.activity)))
  df.mean <- stats::aggregate(. ~ clusters, df, mean)
  zscores <- apply(df.mean[, -1], 2, scale)
  df.mean <- data.frame(clusters = df.mean[, 1],
                        as.data.frame(zscores))
  df.plot <- suppressMessages(reshape2::melt(df.mean,
                                             id.variable = "clusters",
                                             variable.name = "tf",
                                             value.name = "relative_activity"))

  # merge logFC, FDR and mean activity
  df.plot <- merge(df.plot, markers, by = c("tf","clusters"))
  df.plot$tf <- factor(as.character(df.plot$tf), levels = levels)

  # generate bubble plots
  if (bubblesize == "log.FDR") {
    g <- ggplot(df.plot, aes(clusters, tf, color = relative_activity)) +
      geom_point(stat = "identity", aes(size = -log.FDR)) +
      scale_color_viridis_c(option = color.theme) +
      scale_size_continuous("-log.FDR", range = c(0, 7)) +
      theme_classic(base_size = 12) +
      theme(axis.text.x = element_text(angle = 45,
                                       hjust = 1)) +
      labs(color = legend.label) +
      ylab(y.label) + xlab(x.label) + ylab(y.label) +
      xlab(x.label) + ggtitle(title)

  } else if (bubblesize == "summary.logFC") {
    g <- ggplot(df.plot, aes(clusters, tf, color = relative_activity)) +
      geom_point(stat = "identity", aes(size = summary.logFC)) +
      scale_color_viridis_c(option = color.theme) +
      scale_size_continuous("summary logFC", range = c(0,7)) +
      theme_classic(base_size = 12) +
      theme(axis.text.x = element_text(angle = 45,
                                       hjust = 1)) +
      labs(color = legend.label) +
      ylab(y.label) + xlab(x.label) + ggtitle(title)

  } else if (bubblesize == "summary.diff") {
    g <- ggplot(df.plot, aes(clusters, tf, color = relative_activity)) +
      geom_point(stat = "identity", aes(size = summary.diff)) +
      scale_color_viridis_c(option = color.theme) +
      scale_size_continuous("summary diff", range = c(0,7)) +
      theme_classic(base_size = 12) +
      theme(axis.text.x = element_text(angle = 45,
                                       hjust = 1)) +
      labs(color = legend.label) +
      ylab(y.label) + xlab(x.label) + ggtitle(title)
  }
  return(g)
}

#' @importFrom ggplot2 ggplot aes scale_colour_gradient geom_point
#' @importFrom ggplot2 coord_flip theme_bw ggtitle ylab theme unit element_blank

enrichPlot_ <- function(results, title, top) {
    results$logP.adj <- -log10(results$p.adjust)
    ggplot(results[seq_len(top), ], aes(y = logP.adj,
        x = Description, color = GeneRatio)) +
        scale_colour_gradient(high = "red", low = "blue") +
        geom_point(stat = "identity", aes(size = Odds.Ratio)) +
        coord_flip() + theme_bw() + ggtitle(title) +
        ylab(expression("-log"[10] ~ "pval")) +
        theme(text = element_text(size = 10),
            axis.text = element_text(size = 8),
            axis.text.x = element_text(angle = 45,
                hjust = 1), plot.title = element_text(hjust = 0.5),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.spacing = unit(0.5, "lines"))
}

#' Plot results of regulonEnrich
#'
#' @param results  Output from regulonEnrich
#' @param top An integer to indicate the number of pathways to plot ranked by significance. Default is 15.
#' @param ncol An integer to indicate the number of columns in the combined plot, if combine == TRUE. Default is 3.
#' @param title String indicating the title of the combined plot
#' @param combine logical to indicate whether to combine and visualize the plots in one panel
#'
#' @return A combined ggplot object or a list of ggplots if combine == FALSE
#' @export

#' @examples
#' #retrieve genesets
#' msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
#'
#' #convert genesets to be compatible with enricher
#' msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) %in% c("c6", "h")}))]
#' gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
#'  data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))}))
#'
#' head(gs.list)
#'
#' #get regulon
#' library(dorothea)
#' data(dorothea_hs, package = 'dorothea')
#' regulon <- dorothea_hs
#' enrichment_results <- regulonEnrich(c('ESR1','AR'), regulon = regulon, weight = 'mor',
#' genesets = gs.list)
#'
#' # plot graph
#' enrichPlot(results = enrichment_results )
#'
#' @author Xiaosai Yao
enrichPlot <- function(results, top = 15, ncol = 3, title = NULL,
    combine = TRUE) {

    gs <- lapply(names(results), function(x) {
        enrichPlot_(results[[x]], x, top)
    })

    if (combine == TRUE) {

        gs <- patchwork::wrap_plots(gs, ncol = ncol) +
            patchwork::plot_annotation(title = title,
                theme = theme(plot.title = element_text(hjust = 0.5)))

        return(gs)

    } else {

        return(gs)

    }

}

#' Plot targets genes of transcription factors in regulons
#'
#' @param sce A SingleCellExperiment object containing information of cell attributes
#' @param tfs A character vector indicating the names of the transcription factors to be plotted
#' @param regulon A dataframe of regulons containing `tf`, `targets` and a column for filtering the regulons
#' @param regulon_column String indicating the column names to be used for filtering regulons
#' @param regulon_cutoff A scalar indicating the minimal value to retain the regulons for plotting
#' @param downsample Integer indicating the number of cells to sample from the matrix
#' @param scale Logical indicating whether to scale the heatmap
#' @param center Logical indicating whether to center the heatmap
#' @param color_breaks A vector indicating numeric breaks as input to `circlize::colorRamp2`
#' @param colors A vector of colors corresponding to values in `breaks` as input to `circlize::colorRamp2`
#' @param cell_attributes A character vector matching the column names of `colData(sce)` to be used for plotting
#' @param col_gap String indicating the cell attribute to split the columns of the heatmap by
#' @param exprs_values A string specifying which assay in `assays(object)` to obtain expression values from
#' @param use_raster Logical indicating whether to use rasterization to reduce image size
#' @param raster_quality Integer indicating the raster quality. The higher the value, the better the resolution
#' @param cluster_rows Logical indicating whether to cluster rows
#' @param cluster_columns Logical indicating whether to cluster columns
#' @param border Logical indicating whether to add border around heatmap
#' @param show_column_names Logical indicating whether to show column names
#' @param column_col A list specifying the colors in the columns.
#' See [here](https://jokergoo.github.io/ComplexHeatmap-reference/book/heatmap-annotations.html#simple-annotation)
#' @param row_col A list specifying the colors in the rows.
#' See [here](https://jokergoo.github.io/ComplexHeatmap-reference/book/heatmap-annotations.html#simple-annotation)
#' @param genes_label A character vector indicating a selected list of genes to show on the rownames
#' @param ... other arguments for `ComplexHeatmap::Heatmap`
#' @return A Heatmap-class object.
#' @export
#' @examples
#' example_sce <- scuttle::mockSCE()
#' example_sce <- scuttle::logNormCounts(example_sce)
#' example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
#' regulon <- data.frame(tf=c(rep('Gene_0001',10),rep('Gene_0002',20)),
#' target = sample(rownames(example_sce),30), weight = rnorm(30))
#' #plot heatmap and rotate labels
#' plotHeatmapRegulon(example_sce, tfs=c('Gene_0001','Gene_0002'), regulon=regulon,
#' cell_attributes='cluster', col_gap = 'cluster', column_title_rot = 90)
#' @author Xiaosai Yao



plotHeatmapRegulon <- function(sce,
                               tfs, regulon, regulon_column = "weight",
                               regulon_cutoff = 0.1,
                               downsample = 1000,
                               scale = TRUE,
                               center = TRUE,
                               color_breaks = c(-2,
                                                0, 2),
                               colors = c("blue","white", "red"),
                               cell_attributes, col_gap = NULL,
                               exprs_values = "logcounts",
                               use_raster = TRUE,
                               raster_quality = 10,
                               cluster_rows = FALSE,
                               cluster_columns = FALSE,
                               border = TRUE, show_column_names = FALSE,
                               column_col = NULL,
                               row_col = NULL,
                               genes_label = NULL,...) {

  downsample_seq <- seq(from = 1,
                        to = ncol(sce),
                        by = floor(max(1,
                                       ncol(sce)/downsample)))

  # keep only targets belonging to TFs and meeting cutoff
  if (is.matrix(regulon[[regulon_column]])) {

    regulon <- regulon[which(regulon$tf %in% tfs &
                               apply(regulon[[regulon_column]], 1,
                                     function(x) any(x > regulon_cutoff))), ]

  } else {
    regulon <- regulon[regulon$tf %in% tfs &
                         regulon[,regulon_column] > regulon_cutoff, ,drop=FALSE]
  }

  regulon.split <- S4Vectors::split(regulon, f <- regulon$tf)

  # remove duplicated genes from each tf
  regulon.split <- lapply(regulon.split, function(x) x[!duplicated(x$target), ])

  regulon <- do.call(rbind, as.list(regulon.split))

  # remove targets not found in sce
  regulon <- regulon[regulon$target %in% rownames(sce),]
  targets <- regulon$target

  sce <- sce[targets, downsample_seq]


  right_annotation <- data.frame(tf = regulon$tf)
  top_annotation <- data.frame(colData(sce)[cell_attributes])

  if (!is.null(col_gap)) {
    column_split <- top_annotation[col_gap]
  } else {
    column_split <- NULL
  }



  mat <- as.matrix(SummarizedExperiment::assay(sce,
                                               exprs_values))
  mat <- t(scale(t(mat),
                 scale = scale,
                 center = center))

  col_fun <- circlize::colorRamp2(color_breaks,
                                  colors)
  if (!is.null(genes_label)){

    indexmatch <- match(genes_label, rownames(mat))

    ComplexHeatmap::Heatmap(mat,
                            col = col_fun,
                            top_annotation = ComplexHeatmap::HeatmapAnnotation(df = top_annotation,
                                                                               col = column_col),
                            right_annotation = ComplexHeatmap::rowAnnotation(df = right_annotation,
                                                                             col = row_col,
                                                                             foo = ComplexHeatmap::anno_mark(at = stats::na.omit(indexmatch),
                                                                                                             labels = rownames(mat)[stats::na.omit(indexmatch)])),
                            row_split = right_annotation,
                            column_split = column_split,
                            use_raster = use_raster,
                            raster_quality = raster_quality,
                            cluster_rows = cluster_rows,
                            cluster_columns = cluster_columns,
                            border = border,
                            show_column_names = show_column_names,
                            ...)

  } else {
    ComplexHeatmap::Heatmap(mat,
                            col = col_fun,
                            top_annotation = ComplexHeatmap::HeatmapAnnotation(df = top_annotation,
                                                                               col = column_col),
                            right_annotation = ComplexHeatmap::rowAnnotation(df = right_annotation,
                                                                             col = row_col),
                            row_split = right_annotation,
                            column_split = column_split,
                            use_raster = use_raster,
                            raster_quality = raster_quality,
                            cluster_rows = cluster_rows,
                            cluster_columns = cluster_columns,
                            border = border,
                            show_column_names = show_column_names,
                            ...)
  }

}

#' Plot transcription factor activity
#'
#' @param activity_matrix A matrix of values, such as TF activities inferred from calculateActivity
#' @param sce A SingleCellExperiment object containing information of cell attributes
#' @param tfs A character vector indicating the names of the transcription factors to be plotted
#' @param downsample Integer indicating the number of cells to sample from the matrix
#' @param scale Logical indicating whether to scale the heatmap
#' @param center Logical indicating whether to center the heatmap
#' @param color_breaks A vector indicating numeric breaks as input to `circlize::colorRamp2`
#' @param colors A vector of colors corresponding to values in `breaks` as input to `circlize::colorRamp2`
#' @param cell_attributes A character vector matching the column names of `colData(sce)` to be used for plotting
#' @param col_gap String indicating the cell attribute to split the columns of the heatmap by
#' @param use_raster Logical indicating whether to use rasterization to reduce image size
#' @param raster_quality Integer indicating the raster quality. The higher the value, the better the resolution
#' @param cluster_rows Logical indicating whether to cluster rows
#' @param cluster_columns Logical indicating whether to cluster columns
#' @param border Logical indicating whether to add border around heatmap
#' @param show_column_names Logical indicating whether to show column names
#' @param ... other arguments for `ComplexHeatmap::Heatmap`
#' @return A Heatmap-class object.
#' @export
#' @examples
#' example_sce <- scuttle::mockSCE()
#' example_sce <- scuttle::logNormCounts(example_sce)
#' example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
#' activity_matrix <- matrix(rnorm(10*200), nrow=10, ncol=200)
#' rownames(activity_matrix) <- sample(rownames(example_sce),10)
#' plotHeatmapActivity(activity_matrix=activity_matrix, sce=example_sce,
#' tfs=rownames(activity_matrix), cell_attributes='cluster', col_gap='cluster')
#' @author Xiaosai Yao
plotHeatmapActivity <- function(activity_matrix, sce, tfs, downsample = 1000,
    scale = TRUE, center = TRUE, color_breaks = c(-2, 0, 2),
    colors = c("blue", "white", "red"), cell_attributes = NULL,
    col_gap = NULL, use_raster = TRUE, raster_quality = 10,
    cluster_rows = TRUE, cluster_columns = FALSE, border = TRUE,
    show_column_names = FALSE, ...) {

    tfs <- tfs[tfs %in% rownames(activity_matrix)]
    downsample_seq <- seq(from = 1, to = ncol(sce), by = floor(max(1,
        ncol(sce)/downsample)))

    sce <- sce[, downsample_seq]
    top_annotation <- data.frame(colData(sce)[cell_attributes])

    if (!is.null(col_gap)) {
        column_split <- top_annotation[col_gap]
    } else {
        column_split <- NULL
    }

    activity_matrix <- activity_matrix[tfs, downsample_seq]
    activity_matrix <- Matrix::t(scale(Matrix::t(activity_matrix),
        scale = scale, center = center))

    col_fun <- circlize::colorRamp2(color_breaks, colors)

    ComplexHeatmap::Heatmap(activity_matrix, col = col_fun,
        top_annotation = ComplexHeatmap::HeatmapAnnotation(df = top_annotation),
        column_split = top_annotation[col_gap], use_raster = use_raster,
        raster_quality = raster_quality, cluster_rows = cluster_rows,
        cluster_columns = cluster_columns, border = border,
        show_column_names = show_column_names, ...)

}

#' @importFrom ggplot2 ggplot geom_boxplot geom_point aes ggtitle
plotDiagnostic <- function(idx, regulon, expMatrix, exp_assay,
    exp_cutoff = 1, peakMatrix, peak_assay, peak_cutoff = 0,
    clusters) {

    target <- regulon$target[idx]
    tf <- regulon$tf[idx]
    peak <- regulon$idxATAC[idx]

    tg_plot <- list()
    for (cluster in unique(clusters)) {
        target_exp <- SummarizedExperiment::assay(expMatrix,
            exp_assay)[target, clusters == cluster]
        tf_exp <- SummarizedExperiment::assay(expMatrix, exp_assay)[tf,
            clusters == cluster]
        peak_accessibility <- SummarizedExperiment::assay(peakMatrix,
            peak_assay)[peak, clusters == cluster]

        if (is.null(exp_cutoff)) {
            exp_cutoff <- mean(tf_exp)
        }

        if (is.null(peak_cutoff)) {
            peak_cutoff <- mean(peak_accessibility)
        }
        tf_exp.bi <- binarize(tf_exp, cutoff = exp_cutoff)
        peak.bi <- binarize(peak_accessibility, cutoff = peak_cutoff)
        tf_re.bi <- tf_exp.bi * peak.bi
        tf_re.bi <- factor(tf_re.bi, levels = c(0, 1))

        target_group <- data.frame(groups = as.vector(tf_re.bi),
            target = as.vector(target_exp))
        tg_plot[[cluster]] <- ggplot(target_group, aes(groups,
            target)) + geom_boxplot() + geom_point(position = "jitter") +
            ggtitle(paste0("target:", target, " tf:", tf, " peak:",
                peak, " in ", cluster, "\n", " GRN corr:",
                round(regulon$corr[idx, cluster], 2), " GRN pval:",
                round(regulon$pval[idx, cluster], 2), " weight:",
                round(regulon$weight[idx, cluster], 2)))

    }

    tg_plot


}

binarize <- function(input_vector, cutoff) {
    filtered <- as.numeric(input_vector > cutoff)
}

normalizeCols <- function(mat = NULL, scaleTo = NULL) {
    colSm <- Matrix::colSums(mat)
    if (!is.null(scaleTo)) {
        mat@x <- scaleTo * mat@x/rep.int(colSm, Matrix::diff(mat@p))
    } else {
        mat@x <- mat@x/rep.int(colSm, Matrix::diff(mat@p))
    }
    return(mat)
}

#' Test for differential TF activity between pairs of single cell clusters/groups
#'
#' @param activity_matrix A matrix of TF activities inferred from calculateActivity
#' @param clusters A character or integer vector of cluster or group labels for single cells
#' @param test.type String indicating the type of statistical tests to be passed to scran::findMarkers, can be "t", "wilcox". or "binom"
#' @param pval.type A string specifying how p-values are to be combined across pairwise comparisons for a given group/cluster. For more details see \link[scran]{combineMarkers}.
#' @param direction A string specifying direction of differential TF activity, can be "any", "up" or "down"
#' @param logvalues logical indicating whether activities are computed from logged gene expression or not.
#' If activity is computed from linear values of gene expression, setting logvalues to FALSE will return the difference.
#' If activity is computed from logged values of gene expression, setting logvalues to TRUE will return the log changes.
#' @param ... Further arguments to pass to scran::findMarkers
#'
#' @return A named list of dataframes containing differential TF activity test results for each cluster/group
#'
#' @export
#'
#' @examples
#' set.seed(1)
#' score.combine <- cbind(matrix(runif(2000,0,2), 20,100), matrix(runif(2000,0,10), 20,100))
#' rownames(score.combine) <- paste0("TF",1:20)
#' colnames(score.combine) <- paste0("cell",1:200)
#' cluster <- c(rep(1,100),rep(2,100))
#' markers <- findDifferentialActivity(
#' activity_matrix = score.combine,
#' clusters = cluster,
#' pval.type = "some",
#' direction = "up",
#' test.type = "t")
#' sig.genes <- getSigGenes(markers, fdr_cutoff = 1, summary_cutoff = 0.1)
#' @author Xiaosai Yao, Shang-yang Chen

findDifferentialActivity <- function(activity_matrix,
                                     clusters,
                                     test.type = c("t", "wilcox", "binom"),
                                     pval.type = c("some", "any", "all"),
                                     direction = c("any","up","down"),
                                     logvalues = TRUE,
                                     ...){

  test.type = match.arg(test.type)
  pval.type = match.arg(pval.type)
  direction <- match.arg(direction)
  all_tfs <- rownames(activity_matrix)
  activity_matrix <- stats::na.omit(as.matrix(activity_matrix))
  filtered_tfs <- rownames(activity_matrix)
  if(!identical(all_tfs, filtered_tfs)){
    warning(sprintf("The follwing transciption factors are exclded due to the NA values in the data matrix: "),
            paste(setdiff(all_tfs, filtered_tfs), collapse=", "))
  }
  tf_markers <- scran::findMarkers(activity_matrix, clusters, test.type=test.type,
                                   pval.type=pval.type, direction=direction,
                                   sorted = FALSE, ...)


  if (!isTRUE(logvalues)){

    for (cluster in unique(clusters)) {

      # replace logFC with diff
      colnames(tf_markers[[cluster]]) <- gsub("logFC", "diff", colnames(tf_markers[[cluster]]))
    }
  }
  return(tf_markers)

}

#' Compile and summarize the output from findDifferentialActivity function
#'
#' @param da_list List of dataframes from running findDifferentialActivity
#' @param fdr_cutoff A numeric scalar to specify the cutoff for FDR value. Default is 0.05
#' @param summary_cutoff A numeric scalar to specify the cutoff for log fold change or difference
#' @param topgenes A integer scalar to indicate the number of top ordered genes to include in output
#' @param direction A string specifying direction for which differential TF activity was calculated, can be "any", "up" or "down"
#'
#' @return A compiled dataframe of TFs with differential activities across clusters/groups
#' @export
#'
#' @examples
#' set.seed(1)
#' score.combine <- cbind(matrix(runif(2000,0,2), 20,100), matrix(runif(2000,0,10), 20,100))
#' rownames(score.combine) <- paste0("TF",1:20)
#' colnames(score.combine) <- paste0("cell",1:200)
#' cluster <- c(rep(1,100),rep(2,100))
#' markers <- findDifferentialActivity(score.combine, cluster, pval.type = "some", direction = "up",
#' test.type = "t")
#' sig.genes <- getSigGenes(markers, fdr_cutoff = 1, summary_cutoff = 0.1)
#' utils::head(sig.genes)
#' @author Xiaosai Yao, Shang-yang Chen

getSigGenes <- function(da_list,
                        fdr_cutoff = 0.05,
                        summary_cutoff = NULL,
                        topgenes = NULL,
                        direction = c("any","up","down")){

  direction <- match.arg(direction)
  classes <- names(da_list)

  if (direction %in% c("up","down")) {
    direction_factor <- c(down = -1, up = 1)[direction]}

  top.list <- lapply(seq_along(da_list), function(i){
    da_genes <- as.data.frame(da_list[[i]])
    p.value.name <- colnames(da_genes)[grep("p.value", colnames(da_genes))]
    FDR.name <- colnames(da_genes)[grep("FDR", colnames(da_genes))]
    summary.name <- colnames(da_genes)[grep("summary", colnames(da_genes))]
    da_genes <- da_genes[,c(p.value.name, FDR.name,  summary.name)]

    if (is.null(summary_cutoff)){

      if (direction %in% c("up","down")) {
        summary_cutoff <- signif(stats::quantile(da_genes[,summary.name]*direction_factor, 0.95, na.rm=TRUE), digits = 2 )
      } else{
        summary_cutoff <- signif(stats::quantile(abs(da_genes[,summary.name]), 0.95, na.rm=TRUE), digits = 2)
      }
    }else {
      summary_cutoff <- summary_cutoff

    }

    message ("Using a cutoff of ", summary_cutoff, " for class ", classes[i], " for direction equal to ", direction)
    if (direction %in% c("up","down")) {
      da_genes <- da_genes[which(da_genes[,FDR.name] < fdr_cutoff &
                                   da_genes[, summary.name]*direction_factor > summary_cutoff), ]
    } else {
      da_genes <- da_genes[which(da_genes[,FDR.name] < fdr_cutoff &
                                   abs(da_genes[, summary.name]) > summary_cutoff), ]
    }


    if (nrow(da_genes) != 0){
      da_genes$class <- classes[[i]]
      da_genes$tf <- rownames(da_genes); rownames(da_genes) <- NULL
    }

    if (is.null(topgenes)){
      da_genes <- da_genes[order(da_genes[,FDR.name], -(da_genes[, 3])),]
    } else {
      da_genes <- da_genes[utils::head(order(da_genes[,FDR.name], -(da_genes[, 3])), topgenes),]
    }


    return(da_genes)
  })


  return(do.call(rbind, top.list))

}


regulonEnrich_ <- function(TF, regulon, corr, corr_cutoff, genesets) {
  message(TF)
  regulon.TF <- unique(regulon$target[which(regulon$tf ==
                                              TF & regulon[, corr] >
                                              corr_cutoff)])
  if (length(intersect(regulon.TF, genesets$genes)) < 3 | all(!genesets$genes %in% regulon.TF)) {
    results <- data.frame(p.adjust = NA,
                          Description = NA, GeneRatio = 0,
                          Odds.Ratio = NA)
  } else {
    enrichresults <- clusterProfiler::enricher(regulon.TF,
                                               TERM2GENE = genesets)

    results <- enrichresults@result
    results$GeneRatio <- (as.numeric(lapply(strsplit(results$GeneRatio, split = "/"), "[", 1)))/
      (as.numeric(lapply(strsplit(results$GeneRatio, split = "/"), "[", 2)))
    results$BgRatio <- (as.numeric(lapply(strsplit(results$BgRatio, split = "/"), "[", 1)))/
      (as.numeric(lapply(strsplit(results$BgRatio, split = "/"), "[", 2)))
    results$Odds.Ratio <- results$GeneRatio/results$BgRatio
    results <- results[order(results$p.adjust),]
    results$Description <- factor(as.character(results$Description),
                                  levels = unique(as.character(results$Description[nrow(results):1])))
  }
  return(results)
}


#' Perform geneset enrichment of user-defined regulons
#'
#' @param TF A character vector of TF names
#' @param regulon A matrix of weighted regulon consisting of tf, targets, corr and weight
#' @param weight String indicating the column name that should be used to filter target genes for geneset enrichment. Default is 'weight'.
#' @param weight_cutoff A numeric scalar to indicate the cutoff to filter on the column specified by `weight`. Default is 0.5.
#' @param genesets A dataframe with the first column being the name of the geneset and the second column being the name of the genes
#'
#' @return A dataframe showing the significantly enriched pathways
#' @export
#'
#' @examples
#' #retrieve genesets
#' msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
#'
#' #convert genesets to be compatible with enricher
#' msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) %in% c("c6", "h")}))]
#' gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
#'  data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))}))
#'
#' head(gs.list)
#'
#' #get regulon
#' library(dorothea)
#' data(dorothea_hs, package = 'dorothea')
#' regulon <- dorothea_hs
#' enrichment_results <- regulonEnrich(c('ESR1','AR'), regulon = regulon, weight = 'mor',
#' genesets = gs.list)
#'
#' @author Xiaosai Yao

regulonEnrich <- function(TF, regulon, weight = "weight", weight_cutoff = 0.5,
                          genesets) {
  regulonls <- lapply(TF, function(x) {
    regulonEnrich_(x, regulon, weight, weight_cutoff, genesets)
  })
  names(regulonls) <- TF
  return(regulonls)
}

set.seed(102001)
n.cells <- 300
n.tf <- 50
n.groups <- 3
mu <- rnorm(n.tf*3, 1, runif(n.tf*3))
X <- unlist(lapply(mu, function(x) rnorm(100,x,runif(1))))
X.mat <- matrix(X, nrow=n.tf, ncol=n.cells, byrow = TRUE)
rownames(X.mat) <- paste0("gene_", 1:50)
da_genes <- findDifferentialActivity(X.mat, clusters = rep(1:3, each=100), direction = "up")
res <- list()
for(i in 1:3){
  threshold_logFC <- round(quantile(da_genes[[i]][,"summary.logFC"], 0.95),1)
  res[[i]] <- da_genes[[i]][da_genes[[i]][,"summary.logFC"]>threshold_logFC & da_genes[[i]][,"FDR"]<0.05,]
  res[[i]] <- as.data.frame(res[[i]][,c("p.value","FDR","summary.logFC")])
  res[[i]]$class <- as.character(i)
  res[[i]]$tf <- rownames(res[[i]])
  rownames(res[[i]]) <- NULL
  res[[i]] <- res[[i]][order(res[[i]]$FDR, -(res[[i]][, 3])),]
}

res <- do.call(rbind, res)

test_that("getSigGenes works correctly", {
  expect_identical(res, getSigGenes(da_genes, direction = "up"))
})

da_genes <- findDifferentialActivity(X.mat, clusters = rep(1:3, each=100), direction = "down")
res <- list()
for(i in 1:3){
  threshold_logFC <- round(quantile(da_genes[[i]][,"summary.logFC"], 0.05),1)
  res[[i]] <- da_genes[[i]][da_genes[[i]][,"summary.logFC"]<threshold_logFC & da_genes[[i]][,"FDR"]<0.05,]
  res[[i]] <- as.data.frame(res[[i]][,c("p.value","FDR","summary.logFC")])
  if(nrow(res[[i]]) == 0) next
  res[[i]]$class <- as.character(i)
  res[[i]]$tf <- rownames(res[[i]])
  rownames(res[[i]]) <- NULL
  res[[i]] <- res[[i]][order(res[[i]]$FDR, res[[i]][, 3]),]
}

res <- do.call(rbind, res)

test_that("getSigGenes works correctly when direction is 'down'", {
  expect_identical(res, getSigGenes(da_genes, direction="down"))
})



set.seed(4563)
genesets <- data.frame(gs = c(rep("geneset_1", 50), rep("geneset_2", 70), rep("geneset_3", 30)),
                       genes = c(paste0("gene_", 1:50), paste0("gene_", sample(50, 25)),
                                 paste0("gene_", 51:95), paste0("gene_", sample(1:95,15)),
                                 paste0("gene_", 96:110)))

regulon <- data.frame(tf = c(rep("A", 130), rep("B", 200)), target = paste0("gene_", 111:440))
regulon <- rbind(regulon, data.frame(tf= rep("A", 20), target = paste0("gene_", sample(1:50, 20))))
regulon <- rbind(regulon, data.frame(tf = rep("A", 10), target = paste0("gene_", sample(96:110, 10))))
regulon <- rbind(regulon, data.frame(tf = rep("B", 40), target = paste0("gene_", sample(51:95, 40))))
regulon$weight <- runif(nrow(regulon))

enrichresults_1 <- clusterProfiler::enricher(regulon[regulon$tf=="A" & regulon$weight > 0.25,"target"], TERM2GENE = genesets)
enrichresults_2 <- clusterProfiler::enricher(regulon[regulon$tf=="B" & regulon$weight > 0.25,"target"], TERM2GENE = genesets)

res <- list ()
for(enrich_results in list(enrichresults_1, enrichresults_2)){
  results <- enrich_results@result
  results$GeneRatio <- (as.numeric(lapply(strsplit(results$GeneRatio,
                                                   split = "/"), "[",
                                          1)))/(as.numeric(lapply(strsplit(results$GeneRatio,
                                                                           split = "/"), "[",
                                                                  2)))
  results$BgRatio <- (as.numeric(lapply(strsplit(results$BgRatio,
                                                 split = "/"), "[",
                                        1)))/(as.numeric(lapply(strsplit(results$BgRatio,
                                                                         split = "/"), "[",
                                                                2)))
  results$Odds.Ratio <- results$GeneRatio/results$BgRatio
  results <- results[order(results$p.adjust),
  ]
  results$Description <- factor(as.character(results$Description),
                                levels = unique(as.character(results$Description[nrow(results):1])))
  res <- c(res, list(results))

}

names(res) <- c("A", "B")

test_that("regulonEnrich works correctly", {
  expect_equal(regulonEnrich(LETTERS[1:2], regulon, weight = "weight", weight_cutoff = 0.25,
                             genesets), res)
})


set.seed(100)
n_cells = 2000
n_tfs <- 100
n_clusters <- 5
clusters <- sample(letters[1:n_clusters], 2000, replace = TRUE)
unique_clusters <- unique(clusters)
tf_names <- paste0("TF_", 1:n_tfs)
tf_names <- sample(tf_names) # random order
tf_to_cluster_matrix <- matrix(1, nrow=n_tfs, ncol=n_cells)
for(i in seq_len(n_tfs)){
  tf_to_cluster_matrix[i,which(clusters==sample(unique_clusters,1))] <- 2
}
activity_matrix <- matrix(nrow=n_tfs, ncol=n_cells)
activity_matrix[1:length(activity_matrix)] <- rnorm(length(activity_matrix), as.numeric(tf_to_cluster_matrix), 1)

colnames(activity_matrix) <- paste0("cell_", 1:n_cells)
rownames(activity_matrix) <- tf_names
tf_markers <- scran::findMarkers(activity_matrix, clusters, test.type="t",
                                 pval.type="some", direction="up", sorted=FALSE)

test_that("findDifferentialActivity preserves the gene order", {
  expect_equal(rownames(findDifferentialActivity(activity_matrix, clusters, test.type="t")[[1]]), tf_names)
  expect_equal(rownames(findDifferentialActivity(activity_matrix, clusters, test.type="binom")[[1]]), tf_names)
  expect_equal(rownames(findDifferentialActivity(activity_matrix, clusters, test.type="wilcox")[[1]]), tf_names)
})

## ----results='hide', message=FALSE, eval=FALSE--------------------------------
#  if (!require("BiocManager", quietly = TRUE))
#      install.packages("BiocManager")
# 
#  BiocManager::install("epiregulon.extra")
# 

## -----------------------------------------------------------------------------
# load the MAE object
library(scMultiome)

mae <- scMultiome::reprogramSeq()

# expression matrix
GeneExpressionMatrix <- mae[["GeneExpressionMatrix"]]
rownames(GeneExpressionMatrix) <- rowData(GeneExpressionMatrix)$name

reducedDim(GeneExpressionMatrix, "UMAP_Combined") <- 
  reducedDim(mae[["TileMatrix500"]], "UMAP_Combined")

# arrange hash_assignment
GeneExpressionMatrix$hash_assignment <- factor(as.character(
  GeneExpressionMatrix$hash_assignment),
  levels = c("HTO10_GATA6_UTR", "HTO2_GATA6_v2", "HTO8_NKX2.1_UTR", "HTO3_NKX2.1_v2",
    "HTO1_FOXA2_v2", "HTO4_mFOXA1_v2", "HTO6_hFOXA1_UTR", "HTO5_NeonG_v2"
  )
)

## ----calculateActivity, results = "hide"--------------------------------------
library(epiregulon)
library(epiregulon.extra)

data(regulon)

score.combine <- calculateActivity(
  expMatrix = GeneExpressionMatrix,
  regulon = regulon,
  mode = "weight",
  method = "weightedMean",
  exp_assay = "normalizedCounts",
  normalize = FALSE
)

## ----differential-------------------------------------------------------------
library(epiregulon.extra)
markers <- findDifferentialActivity(
  activity_matrix = score.combine,
  clusters = GeneExpressionMatrix$hash_assignment,
  pval.type = "some",
  direction = "up",
  test.type = "t",
  logvalue = FALSE
)

## -----------------------------------------------------------------------------
markers.sig <- getSigGenes(markers, topgenes = 5)

## ----visualization------------------------------------------------------------
plotBubble(
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2"),
  clusters = GeneExpressionMatrix$hash_assignment
)

## -----------------------------------------------------------------------------
plotBubble(
  activity_matrix = score.combine,
  tf = markers.sig$tf,
  clusters = GeneExpressionMatrix$hash_assignment
)

## -----------------------------------------------------------------------------
plotActivityViolin(
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  clusters = GeneExpressionMatrix$hash_assignment
)

## ----fig.height = 6, fig.width = 9--------------------------------------------

options(ggrastr.default.dpi=100)

ActivityPlot <- plotActivityDim(
  sce = GeneExpressionMatrix,
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  dimtype = "UMAP_Combined",
  label = "Clusters",
  point_size = 0.3,
  ncol = 3,
  rasterise = TRUE
)


ActivityPlot

## ----fig.height = 6, fig.width = 9--------------------------------------------

options(ggrastr.default.dpi=100)

ActivityPlot <- plotActivityDim(
  sce = GeneExpressionMatrix,
  activity_matrix = counts(GeneExpressionMatrix),
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  dimtype = "UMAP_Combined",
  label = "Clusters",
  point_size = 0.3,
  ncol = 3,
  limit = c(0, 2),
  colors = c("grey", "blue"),
  legend.label = "GEX",
  rasterise = TRUE
)
ActivityPlot

## ----eval=FALSE---------------------------------------------------------------
# plotHeatmapRegulon(
#   sce = GeneExpressionMatrix,
#   tfs = c("GATA6", "NKX2-1"),
#   regulon = regulon,
#   regulon_cutoff = 0,
#   downsample = 1000,
#   cell_attributes = "hash_assignment",
#   col_gap = "hash_assignment",
#   exprs_values = "normalizedCounts",
#   name = "regulon heatmap",
#   column_title_rot = 45,
#   raster_quality=4
# )
# 
# 
# plotHeatmapActivity(
#   activity = score.combine,
#   sce = GeneExpressionMatrix,
#   tfs = c("GATA6", "NKX2-1", "FOXA1", "FOXA2"),
#   downsample = 5000,
#   cell_attributes = "hash_assignment",
#   col_gap = "hash_assignment",
#   name = "Activity",
#   column_title_rot = 45,
#   raster_quality=3
# )

## ----enrichment, fig.height = 8, fig.width = 14-------------------------------
# retrieve genesets
msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) %in% c("c6", "h")}))]

# convert genesets to be compatible with enricher
gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
  data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))
}))

enrichresults <- regulonEnrich(
  TF = c("GATA6", "NKX2-1"),
  regulon = regulon,
  weight = "weight",
  weight_cutoff = 0.1,
  genesets = gs.list
)

# plot results
enrichPlot(results = enrichresults)

## ----plot gsea network--------------------------------------------------------
plotGseaNetwork(
  tf = names(enrichresults),
  enrichresults = enrichresults,
  p.adj_cutoff = 0.1,
  ntop_pathways = 10
)

## ----differential networks----------------------------------------------------
plotDiffNetwork(regulon,
  cutoff = 0.1,
  tf = c("GATA6"),
  weight = "weight",
  clusters = c("C1", "C5"),
  layout = "stress"
)

plotDiffNetwork(regulon,
  cutoff = 0.1,
  tf = c("NKX2-1"),
  weight = "weight",
  clusters = c("C3", "C5"),
  layout = "stress"
)

## -----------------------------------------------------------------------------
selected <- which(regulon$weight[, "C1"] > 0 &
  regulon$tf %in% c("GATA6", "FOXA1", "AR"))

C1_network <- buildGraph(regulon[selected, ],
  weights = "weight",
  cluster = "C1"
  )

plotEpiregulonNetwork(C1_network,
  layout = "sugiyama",
  tfs_to_highlight = c("GATA6", "FOXA1", "AR")) + 
  ggplot2::ggtitle("C1")

## -----------------------------------------------------------------------------
# rank by differential centrality
C1_network <- buildGraph(regulon, weights = "weight", cluster = "C1")
C5_network <- buildGraph(regulon, weights = "weight", cluster = "C5")

diff_graph <- buildDiffGraph(C1_network, C5_network, abs_diff = FALSE)
diff_graph <- addCentrality(diff_graph)
diff_graph <- normalizeCentrality(diff_graph)
rank_table <- rankTfs(diff_graph)

library(ggplot2)
ggplot(rank_table, aes(x = rank, y = centrality)) +
  geom_point() +
  ggrepel::geom_text_repel(data = rank_table, aes(label = tf)) +
  theme_classic()

## -----------------------------------------------------------------------------
sessionInfo()

#' Find interaction partners of a transcription factor of interest
#' @param graph a igraph object from `buildGraph` or `buildDiffGraph`
#' @param focal_tf character string indicating the name of the transcription factors to find
#' interaction partners of
#' @export
#' @return A list with elements corresponding to each transcription factor apart from
#' the focal one. Each list element is represented as a data frame with columns containing
#' names of all target genes shared with focal transcription factor, weights of edges
#' connecting transcription factor with target genes, equivalent weights for focal transcription
#' factor and the element wise product of both weight columns.
#' @examples
#' regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
#' target = sample(letters[5:14], 100, replace = TRUE))
#' regulon$weights <- runif(100)
#' GRN_graph <- buildGraph(regulon)
#' partners <- findPartners(GRN_graph, 'a')
#' @importFrom igraph V E ego vertex_attr edge_attr delete_vertices subgraph shortest_paths similarity intersection subcomponent rewire each_edge ends
findPartners <- function(graph, focal_tf) {
  checkmate::assert_character(focal_tf, len = 1)
  checkmate::assert_class(graph, "igraph")

  if (length(unique(vertex_attr(graph, name = "type"))) != 2)
    stop("Graph should contain vertices of two types. Input graph should not be created with the use of tripartite mode")
  focal_vertex <- V(graph)[V(graph)$name == focal_tf & V(graph)$type == "transcription factor"]
  if (length(focal_vertex) == 0)
    stop(sprintf("Focal transcription factor (%s) is not present in the input graph"),
         focal_tf)
  focal_tf_targets <- ego(graph, nodes = focal_vertex, mode = "out", mindist = 1)[[1]]

  # keep other transcription factors
  redundant_vertices <- setdiff(as.numeric(V(graph)), 
                                as.numeric(V(graph)[V(graph)$type == "transcription factor"]))

  # keep focal tf's target genes
  redundant_vertices <- setdiff(redundant_vertices, as.numeric(focal_tf_targets))

  # prune network removing target genes which do not belong to the focal tf's
  # regulon
  graph <- delete_vertices(graph, redundant_vertices)
  if (any(duplicated(ends(graph, es = E(graph)))))
    stop("Duplicated edges")
  
  # find focal tf in the new graph
  focal_vertex <- V(graph)[V(graph)$name == focal_tf & V(graph)$type == "transcription factor"]
  other_tfs <- V(graph)[V(graph)$type == "transcription factor" & V(graph)$name !=
                          focal_tf]
  all_tfs <- c(focal_vertex, other_tfs)  # focal tf first on the list
  focal_tf_targets <- V(graph)[V(graph)$type == "target gene"]

  # create regulons using focal tf's target genes
  tf_targets_subgraphs <- lapply(all_tfs, function(tf, targets) 
    subgraph(graph, vids = c(tf, intersection(targets, subcomponent(graph, tf, mode = "out")))),
    focal_tf_targets)
  focal_weights <- edge_attr(tf_targets_subgraphs[[1]], index = E(tf_targets_subgraphs[[1]]), name = "weight")
  common_targets <- lapply(setdiff(seq_along(all_tfs), 1), 
                           get_common_targets, all_tfs, tf_targets_subgraphs, focal_weights, focal_tf_targets)
  do.call(c, common_targets)
}

get_common_targets <- function(idx, tfs, tf_targets_subgraphs, focal_weights, focal_tf_targets){
  res_list <- list()
  tf <- V(tf_targets_subgraphs[[idx]])[V(tf_targets_subgraphs[[idx]])$type == "transcription factor" &
                                       V(tf_targets_subgraphs[[idx]])$name == tfs[idx]$name]
  # determine targets shared between focal tf and current tf
  common_targets <- V(tf_targets_subgraphs[[idx]])[V(tf_targets_subgraphs[[idx]])$type ==
                                                   "target gene"]
  # extract edges connection shared targets with the current tf
  edges_to_tf <- shortest_paths(tf_targets_subgraphs[[idx]], from = tf, to = common_targets,
                                output = "epath")
  edges_to_tf <- do.call(c, edges_to_tf)
  # find indices of common targets in the vector of all targets of focal
  # tf
  common_targets_idx <- match(common_targets$name, focal_tf_targets$name)
  res_list[[tf$name]] <- data.frame(target = common_targets$name, focal_weight = focal_weights[common_targets_idx],
                                    other_tf_weight = edge_attr(tf_targets_subgraphs[[idx]],
                                                                        index = edges_to_tf, name = "weight"))  # extract weights from the edges
  res_list[[tf$name]]$weight_product <- res_list[[tf$name]]$focal_weight *
    res_list[[tf$name]]$other_tf_weight
  res_list
}

#' Calculate Jaccard Similarity between regulons of all transcription factors
#' @param graph a igraph object from `buildGraph` or `buildDiffGraph`
#' @importFrom igraph V similarity
#' @return A matrix with Jaccard similarity between all pairs of transcription factors.
#' @export
#' @examples
#' regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
#' target = sample(letters[5:14], 100, replace = TRUE))
#' regulon$weights <- runif(100)
#' GRN_graph <- buildGraph(regulon)
#' similarity <- calculateJaccardSimilarity(GRN_graph)
calculateJaccardSimilarity <- function(graph) {
  all_tfs <- V(graph)[V(graph)$type == "transcription factor"]
  res <- similarity(graph, vids = all_tfs, method = "jaccard", mode = "out")
  rownames(res) <- colnames(res) <- V(graph)[all_tfs]$name
  res
}

#' Calculate similarity score from permuted graphs to estimate background similarity
#' @param graph an igraph object from `buildGraph` or `buildDiffGraph`
#' @param focal_tf character string indicating the name of the transcription factors to
#' calculate similarity score
#' @param n an integer indicating the number of permutations
#' @param p a scalar indicating the probability of rewiring the graphs
#' @return A matrix with Jaccard similarity between the focal transcription factor and all pairs of transcription factors
#' for n permuted graphs
#' @importFrom igraph V rewire
#' @export
#' @examples
#' regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
#' target = sample(letters[5:14], 100, replace = TRUE))
#' regulon$weights <- runif(100)
#' GRN_graph <- buildGraph(regulon)
#' permuted_graph <- permuteGraph(GRN_graph, focal_tf = "a")
permuteGraph <- function(graph, focal_tf, n = 100, p = 1) {
  if (!focal_tf %in% names(V(graph)[V(graph)$type == "transcription factor"]))
    stop(focal_tf, " vertex shoud be present in the graph")
  all_tfs <- names(V(graph)[V(graph)$type == "transcription factor"])
  permute_matrix <- matrix(rep(NA, length(all_tfs) * n),
                           nrow = length(all_tfs))
  rownames(permute_matrix) <- all_tfs

  for (iteration in seq_len(n)) {
    diff_graph_permute <- rewire(graph, each_edge(prob = p))
    similarity_score <- calculateJaccardSimilarity(diff_graph_permute)
    permute_matrix[, iteration] <- similarity_score[focal_tf, all_tfs]
  }
  permute_matrix

}

regulon <- data.frame(tf = rep(LETTERS[1:5], times = c(5,5,6,3,1)), target = NA)
regulon$target[regulon$tf=="A"] <- LETTERS[6:10]
regulon$target[regulon$tf=="B"][c(1,3,5)] <- regulon$target[regulon$tf=="A"][c(1,2,4)]
regulon$target[regulon$tf=="C"][6] <- regulon$target[regulon$tf=="A"][3]
regulon$target[regulon$tf=="E"] <- "B"
regulon$target[is.na(regulon$target)] <- sample(LETTERS[11:15], sum(is.na(regulon$target)), replace = TRUE)
regulon$weights <- runif(nrow(regulon))
regulon$weights <- regulon$weights * sample(c(1,-1), length(regulon$weights),replace = TRUE)

res <- vector(mode = "list", 4)
names(res) <- LETTERS[2:5]
res <- lapply(res, function(x) data.frame(target=character(0), focal_weight=numeric(0),
                                          other_tf_weight=numeric(0), weight_product=numeric(0)))

res$B <- rbind(res$B, data.frame(target=regulon$target[regulon$tf=="A"][c(1,2,4)],
                                 focal_weight=regulon$weights[regulon$tf=="A"][c(1,2,4)],
                                 other_tf_weight=regulon$weights[regulon$tf=="B"][c(1,3,5)],
                                 weight_product=NA))

res$C <- rbind(res$C, data.frame(target=regulon$target[regulon$tf=="A"][3],
                                 focal_weight=regulon$weights[regulon$tf=="A"][3],
                                 other_tf_weight=regulon$weights[regulon$tf=="C"][6],
                                 weight_product=NA))

res <- lapply(res, function(x) {x$weight_product <- x$focal_weight*x$other_tf_weight; x})

test_graph <- buildGraph(regulon)
partners_graph <- findPartners(test_graph, "A")

test_that("findPartners function works correctly", {
  expect_identical(partners_graph[order(names(partners_graph))], res[order(names(res))])
})

set.seed(4201)

regulon <- data.frame(tf = rep(LETTERS[1:7], times = rmultinom(1,200, prob = c(0.02, 0.05, 0.1, 0.15, 0.2, 0.25, 0.28))[,1])[sample(200)],
                      target = c(rep(c("F", "G"),3), paste0("gene_", 1:30)[sample(1:30,194, replace=TRUE)])[sample(200)])

# add tfs disjoint to the previous one in terms of their regulons
regulon <- rbind(regulon, data.frame(tf = LETTERS[9:11][sample(1:3,30,replace = TRUE)],
                      target = paste0("gene_", 31:40)[sample(1:10, 30, replace = TRUE)]))

test_graph <- buildGraph(regulon, weights = NULL)
tfs <- unique(regulon$tf)
tf_n <- length(tfs)
js_matrix <- matrix(0, tf_n, tf_n, dimnames = list(unique(regulon$tf), unique(regulon$tf)))
js_matrix <- js_matrix + diag(1, tf_n, tf_n)

for(i in seq_len(tf_n-1)){
  for(j in i:tf_n)
  js_matrix[i,j] <- js_matrix[j,i] <- length(intersect(regulon[regulon$tf==tfs[i], "target"], regulon[regulon$tf==tfs[j], "target"]))/length(unique(regulon[regulon$tf %in% tfs[c(i,j)], "target"]))
}

tf_order <- V(test_graph)$name[V(test_graph)$type=="transcription factor"]
js_matrix <- js_matrix[tf_order, tf_order]

permute_matrix <- permuteGraph(test_graph, n = 90, focal_tf = V(test_graph)$name[V(test_graph)$type=="transcription factor"][1])

test_that("calculateJaccardSimilarity function works correctly", {
  expect_equal(calculateJaccardSimilarity(test_graph), js_matrix)
})

test_that("permuteGraph function works correctly", {
  expect_identical(dim(permute_matrix), c(9L,90L))
  expect_true(all(apply(permute_matrix,2,sd)[apply(permute_matrix,2,function(x) sum(x)!=0)]>0))
  expect_true(!any(is.na(permute_matrix)))
  expect_identical(rownames(permute_matrix), tfs)
})

#' regulon created using `epiregulon` package from reprogram-seq data
#' @name regulon
#' @docType data
#' @usage data(regulon)
#' @format a DFrame.
#' @return a DFrame.
#' @keywords datasets
#' @examples
#' data(regulon)
"regulon"

#' Plot networks graph of significant genesets from regulonEnrich results
#'
#' @param tf A vector of gene names to be plotted. They should be present in enrichresults
#' @param enrichresults Output from regulonEnrich that computes enriched genesets from user-specified regulons
#' of interest
#' @param ntop_pathways An integer indicating the number of top pathways to be included in the graph
#' @param p.adj_cutoff A scalar indicating the p.adjusted cutoff for pathways to be included in the graph. Default value is 0.05
#' @param layout String indicating layout option from igraph
#' @param tf_label String indicating the name of the tf label
#' @param gset_label String indicating the name of the geneset label
#' @param tf_color String indicating the color of the tf label
#' @param gset_color String indicating the color of the geneset label
#' @return an igraph plot of interconnected pathways through TFs
#' @importFrom igraph graph_from_data_frame layout_with_sugiyama V<- V 
#' @importFrom ggraph geom_edge_link theme_graph geom_node_point geom_node_text
#' @importFrom ggplot2 coord_flip scale_color_manual scale_y_continuous expansion
#' @export
#' @examples
#' AR <- data.frame(ID = c('ANDROGEN RESPONSE','PROLIFERATION','MAPK'),
#' p.adjust = c(0.001, 0.01, 0.04))
#' GATA6 <- data.frame(ID = c('STK33','PROLIFERATION','MAPK'),
#' p.adjust = c(0.001, 0.01, 0.04))
#' enrichresults <- list(AR = AR, GATA6 = GATA6)
#' plotGseaNetwork(tf = names(enrichresults), enrichresults = enrichresults)
#' @author Phoebe Guo, Xiaosai Yao

plotGseaNetwork <- function(tf, enrichresults, ntop_pathways = 10, p.adj_cutoff = 0.05,
                            layout = "sugiyama", tf_label = "tf", gset_label = "ID", tf_color = "tomato",
                            gset_color = "grey") {

  # filter non-significant pathways by p.adj
  enrichresults.filter <- lapply(stats::setNames(names(enrichresults), names(enrichresults)),
                                 function(x) {
                                   enrichresults[[x]][which(as.numeric(enrichresults[[x]][, "p.adjust"]) <
                                                              p.adj_cutoff), ]
                                 })

  # extract top pathways from enrichr results
  pathway.list <- lapply(stats::setNames(tf, tf), function(x) {
    head(enrichresults.filter[[x]][order(as.numeric(enrichresults.filter[[x]][,
                                                                              "p.adjust"])), ], ntop_pathways)
  })


  # filter out empty elements
  pathway.list <- pathway.list[lapply(pathway.list, nrow) > 0]


  # bind rows of pathway list
  pathway.df <- do.call(rbind, lapply(names(pathway.list), function(x) {
    data.frame(tf = x, ID = pathway.list[[x]][, gset_label])
  }))

  # create node object for igraph
  nodes.list <- lapply(names(pathway.list), function(x) {
    data.frame(rbind(data.frame(name = x, type = tf_label),
                     data.frame(name = pathway.list[[x]][,gset_label], type = gset_label)))
  })

  nodes <- unique(do.call(rbind, nodes.list))


  # create the network object
  p <- graph_from_data_frame(d = pathway.df, vertices = nodes, directed = FALSE)

  # Create a vector of color
  col <- c(gset_color, tf_color)
  pal <- col[as.numeric(as.factor(V(p)$type))]
  V(p)$type.num <- as.numeric(factor(V(p)$type, levels = c(tf_label,
                                                                           gset_label)))

  V(p)$color <- pal

  # Create sugiyama layout
  l <- layout_with_sugiyama(p, attributes = "all", layers = V(p)$type.num)

  # plot
  gseaplot <- ggraph(l$extd_graph, layout = "sugiyama", layers = l$layout[,2]) +
    geom_edge_link(alpha = 0.8) + coord_flip() + scale_color_manual(values = pal) +
    theme_graph(base_family = "Helvetica", fg_text_colour = "black") +
    geom_node_point(aes(color = type), size = 5) +
    geom_node_text(aes(label = name, filter = type == gset_label), nudge_y = 0.1, hjust = 0) +
    geom_node_text(aes(label = name, filter = type == tf_label), nudge_y = -0.1, hjust = 1) +
    scale_y_continuous(expand = expansion(add = c(1,4)))
  gseaplot

}

globalVariables(c("head", "name", "type", "logP.adj", "Description",
                  "GeneRatio","Odds.Ratio","activity","relative_activity","summary.logFC",
                  "summary.diff","log.FDR","groups"), package = "epiregulon.extra",
    add = TRUE)

---
title: "Data visualization with epiregulon.extra"
author: "Xiaosai Yao, Tomasz Włodarczyk"
email: "yao.xiaosai@gene.com"
output:
  BiocStyle::html_document:
    toc: true
    number_section: true
    self_contained: true
    titlecaps: true
package: "epiregulon.extra"
vignette: >
  %\VignetteIndexEntry{Data visualization with epiregulon.extra}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction
`r BiocStyle::Biocpkg("epiregulon.extra")` is a companion package to `r BiocStyle::Biocpkg("epiregulon")` and provides functions to visualize transcription factor activity and network. It also provides statistical tests to identify transcription factors with differential activity and network topology. This tutorial continues from the reprogram-seq example included in epiregulon. We will use the gene regulatory networks constructed by epiregulon and continue with visualization and network analysis. 

For full documentation of the `epiregulon` package, please refer to the [epiregulon book](https://xiaosaiyao.github.io/epiregulon.book/).

# Installation

```{r, results='hide', message=FALSE, eval=FALSE}
 if (!require("BiocManager", quietly = TRUE))
     install.packages("BiocManager")

 BiocManager::install("epiregulon.extra")

```

# Data preparation

To continue with the visualization functions, we will first need the gene expression matrix. We can download the data from `r BiocStyle::Biocpkg("scMultiome")`. <br>

```{r}
# load the MAE object
library(scMultiome)

mae <- scMultiome::reprogramSeq()

# expression matrix
GeneExpressionMatrix <- mae[["GeneExpressionMatrix"]]
rownames(GeneExpressionMatrix) <- rowData(GeneExpressionMatrix)$name

reducedDim(GeneExpressionMatrix, "UMAP_Combined") <- 
  reducedDim(mae[["TileMatrix500"]], "UMAP_Combined")

# arrange hash_assignment
GeneExpressionMatrix$hash_assignment <- factor(as.character(
  GeneExpressionMatrix$hash_assignment),
  levels = c("HTO10_GATA6_UTR", "HTO2_GATA6_v2", "HTO8_NKX2.1_UTR", "HTO3_NKX2.1_v2",
    "HTO1_FOXA2_v2", "HTO4_mFOXA1_v2", "HTO6_hFOXA1_UTR", "HTO5_NeonG_v2"
  )
)
```

# Calculate TF activity 

Next we load the regulon object previously calculated by epiregulon. Here we just use the pruned version of regulon object in which only relevant columns are kept. 
Using epiregulon, we can calculate activity of transcription factors included in the regulon object.

```{r calculateActivity, results = "hide"}
library(epiregulon)
library(epiregulon.extra)

data(regulon)

score.combine <- calculateActivity(
  expMatrix = GeneExpressionMatrix,
  regulon = regulon,
  mode = "weight",
  method = "weightedMean",
  exp_assay = "normalizedCounts",
  normalize = FALSE
)
```

# Perform differential activity

We perform a differential analysis of transcription factor activity across groups of cells. This function is a wrapper around `findMarkers` from `r BiocStyle::Biocpkg("scran")`.
```{r differential}
library(epiregulon.extra)
markers <- findDifferentialActivity(
  activity_matrix = score.combine,
  clusters = GeneExpressionMatrix$hash_assignment,
  pval.type = "some",
  direction = "up",
  test.type = "t",
  logvalue = FALSE
)
```

We can specify the top transcription factors of interest
```{r}
markers.sig <- getSigGenes(markers, topgenes = 5)
```
# Visualize the results

We visualize the known differential transcription factors by bubble plot
```{r visualization}
plotBubble(
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2"),
  clusters = GeneExpressionMatrix$hash_assignment
)
```

Then visualize the most differential transcription factors by clusters
```{r}
plotBubble(
  activity_matrix = score.combine,
  tf = markers.sig$tf,
  clusters = GeneExpressionMatrix$hash_assignment
)
```

Visualize the known differential transcription factors by violin plot. 

```{r}
plotActivityViolin(
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  clusters = GeneExpressionMatrix$hash_assignment
)
```

Visualize the known differential transcription factors by UMAP
```{r, fig.height = 6, fig.width = 9}

options(ggrastr.default.dpi=100)

ActivityPlot <- plotActivityDim(
  sce = GeneExpressionMatrix,
  activity_matrix = score.combine,
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  dimtype = "UMAP_Combined",
  label = "Clusters",
  point_size = 0.3,
  ncol = 3,
  rasterise = TRUE
)


ActivityPlot
```

In contrast, the gene expression of the transcription factors is very sparse
```{r, fig.height = 6, fig.width = 9}

options(ggrastr.default.dpi=100)

ActivityPlot <- plotActivityDim(
  sce = GeneExpressionMatrix,
  activity_matrix = counts(GeneExpressionMatrix),
  tf = c("NKX2-1", "GATA6", "FOXA1", "FOXA2", "AR"),
  dimtype = "UMAP_Combined",
  label = "Clusters",
  point_size = 0.3,
  ncol = 3,
  limit = c(0, 2),
  colors = c("grey", "blue"),
  legend.label = "GEX",
  rasterise = TRUE
)
ActivityPlot
```

Visualize the activity of the selected transcription factors by heat map. Due to the maximum size limit for this vignette, the output is not shown here.

```{r eval=FALSE}
plotHeatmapRegulon(
  sce = GeneExpressionMatrix,
  tfs = c("GATA6", "NKX2-1"),
  regulon = regulon,
  regulon_cutoff = 0,
  downsample = 1000,
  cell_attributes = "hash_assignment",
  col_gap = "hash_assignment",
  exprs_values = "normalizedCounts",
  name = "regulon heatmap",
  column_title_rot = 45,
  raster_quality=4
)


plotHeatmapActivity(
  activity = score.combine,
  sce = GeneExpressionMatrix,
  tfs = c("GATA6", "NKX2-1", "FOXA1", "FOXA2"),
  downsample = 5000,
  cell_attributes = "hash_assignment",
  col_gap = "hash_assignment",
  name = "Activity",
  column_title_rot = 45,
  raster_quality=3
)
```

# Geneset enrichment

Sometimes we are interested to know what pathways are enriched in the regulon of a particular TF. We can perform geneset enrichment using the enricher function from `r BiocStyle::Biocpkg("clusterProfiler")`.

```{r enrichment, fig.height = 8, fig.width = 14}
# retrieve genesets
msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) %in% c("c6", "h")}))]

# convert genesets to be compatible with enricher
gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
  data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))
}))

enrichresults <- regulonEnrich(
  TF = c("GATA6", "NKX2-1"),
  regulon = regulon,
  weight = "weight",
  weight_cutoff = 0.1,
  genesets = gs.list
)

# plot results
enrichPlot(results = enrichresults)
```

# Network analysis

We can visualize the genesets as a network
```{r plot gsea network}
plotGseaNetwork(
  tf = names(enrichresults),
  enrichresults = enrichresults,
  p.adj_cutoff = 0.1,
  ntop_pathways = 10
)
```

# Differential networks

We are interested in understanding the differential networks between two conditions and determining which transcription factors account for the differences in the topology of networks. The pruned regulons with cluster-specific test statistics computed by `pruneRegulon` can be used to generate cluster-specific networks based on user-defined cutoffs and to visualize differential networks for transcription factors of interest. In this dataset, the GATA6 gene was only expressed in cluster 1 (C1) and NKX2-1 was only expressed in cluster 3 (C3). If we visualize the target genes of GATA6, we can see that C1 has many more target genes of GATA6 compared to C5, a cluster that does not express GATA6. Similarly, NKX2-1 target genes are confined to C3 which is the only cluster that exogenously expresses NKX2-1.

```{r differential networks}
plotDiffNetwork(regulon,
  cutoff = 0.1,
  tf = c("GATA6"),
  weight = "weight",
  clusters = c("C1", "C5"),
  layout = "stress"
)

plotDiffNetwork(regulon,
  cutoff = 0.1,
  tf = c("NKX2-1"),
  weight = "weight",
  clusters = c("C3", "C5"),
  layout = "stress"
)
```

We can also visualize how transcription factors relate to other transcription factors in C1 cluster.
```{r}
selected <- which(regulon$weight[, "C1"] > 0 &
  regulon$tf %in% c("GATA6", "FOXA1", "AR"))

C1_network <- buildGraph(regulon[selected, ],
  weights = "weight",
  cluster = "C1"
  )

plotEpiregulonNetwork(C1_network,
  layout = "sugiyama",
  tfs_to_highlight = c("GATA6", "FOXA1", "AR")) + 
  ggplot2::ggtitle("C1")
```
To systematically examine the differential network topology between two clusters, we perform an edge subtraction between two graphs, using weights computed by `pruneRegulon`. We then calculate the degree centrality of the weighted differential graphs and if desired, normalize the differential centrality against the total number of edges. The default normalization function is `sqrt` as it preserves both the difference in the number of edges (but scaled by sqrt) and the differences in the weights. If the user only wants to examine the differences in the averaged weights, the `FUN` argument can be changed to `identity`. Finally, we rank the transcription factors by (normalized) differential centrality. For demonstration purpose, the regulon list is truncated but the full list would contain all the transcription factors.


```{r}
# rank by differential centrality
C1_network <- buildGraph(regulon, weights = "weight", cluster = "C1")
C5_network <- buildGraph(regulon, weights = "weight", cluster = "C5")

diff_graph <- buildDiffGraph(C1_network, C5_network, abs_diff = FALSE)
diff_graph <- addCentrality(diff_graph)
diff_graph <- normalizeCentrality(diff_graph)
rank_table <- rankTfs(diff_graph)

library(ggplot2)
ggplot(rank_table, aes(x = rank, y = centrality)) +
  geom_point() +
  ggrepel::geom_text_repel(data = rank_table, aes(label = tf)) +
  theme_classic()
```

# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphs.R
\name{plotEpiregulonNetwork}
\alias{plotEpiregulonNetwork}
\title{Plot a graph build based on \code{getRegulon} output}
\usage{
plotEpiregulonNetwork(
  graph,
  layout = "stress",
  label_size = 3,
  tfs_to_highlight = NULL,
  edge_alpha = 0.02,
  point_size = 1,
  point_border_size = 0.5,
  label_alpha = 0.8,
  label_nudge_x = 0.2,
  label_nudge_y = 0.2,
  ...
)
}
\arguments{
\item{graph}{an igraph object}

\item{layout}{a layout specification. Any values that are valid for
\link[ggraph]{ggraph} or \link[ggraph]{create_layout} will work. Defaults to
'stress'. Consider also trying 'mds', 'nicely', and 'fr' while you experiment.}

\item{label_size}{an integer indicating how large the labels of highlighted
transcription factors should be}

\item{tfs_to_highlight}{a character vector specifying which TFs in the plot
should be highlighted. Defaults to NULL (no labels).}

\item{edge_alpha}{a numeric value between 0 and 1 indicating the level of
transparency to use for the edge links in the force-directed layout. Defaults
to 0.02.}

\item{point_size}{a numeric value indicating the size of nodes in the force-directed layout}

\item{point_border_size}{a numeric value indicating the size of point
borders for nodes in the force-directed layout}

\item{label_alpha}{a numeric value between 0 and 1 indicating the level of
transparency to use for the labels of highlighted nodes}

\item{label_nudge_x}{a numeric value indicating the shift of the labels
along the x axis that should be used in the force-directed layout}

\item{label_nudge_y}{A numeric value indicating the shift of the labels
along the y axis that should be used in the force-directed layout.}

\item{...}{optional additional arguments to pass to \link[ggraph]{create_layout}}
}
\value{
a ggraph object
}
\description{
This function takes an input an igraph object created by any of the following:
\code{buildGraph}, \code{addCentrality}, \code{igraph::strength}, \code{normalizeCentrality}.
It makes a force-directed layout plot to visualize it at a high level.
}
\examples{
# create an artificial getRegulon output
set.seed(1234)
tf_set <- apply(expand.grid(LETTERS[seq_len(5)], LETTERS[seq_len(5)]),1,  paste, collapse = '')
regulon <- data.frame(tf = sample(tf_set, 5e2, replace = TRUE))
gene_set <- expand.grid(LETTERS[seq_len(5)], LETTERS[seq_len(5)], LETTERS[seq_len(5)])
gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
regulon$target <- sample(gene_set, 5e2, replace = TRUE)
regulon$idxATAC <- seq_len(5e2)
regulon$corr <- runif(5e2)*0.5+0.5
regulon$weights <- runif(500)
#create igraph object
graph_tripartite <- buildGraph(regulon, mode = 'tripartite')
plotEpiregulonNetwork(graph_tripartite, tfs_to_highlight = sample(unique(tf_set),3),
edge_alpha = 0.2)
}
\author{
Timothy Keyes, Tomasz Wlodarczyk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulonSimilarities.R
\name{calculateJaccardSimilarity}
\alias{calculateJaccardSimilarity}
\title{Calculate Jaccard Similarity between regulons of all transcription factors}
\usage{
calculateJaccardSimilarity(graph)
}
\arguments{
\item{graph}{a igraph object from \code{buildGraph} or \code{buildDiffGraph}}
}
\value{
A matrix with Jaccard similarity between all pairs of transcription factors.
}
\description{
Calculate Jaccard Similarity between regulons of all transcription factors
}
\examples{
regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
target = sample(letters[5:14], 100, replace = TRUE))
regulon$weights <- runif(100)
GRN_graph <- buildGraph(regulon)
similarity <- calculateJaccardSimilarity(GRN_graph)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotHeatmapActivity}
\alias{plotHeatmapActivity}
\title{Plot transcription factor activity}
\usage{
plotHeatmapActivity(
  activity_matrix,
  sce,
  tfs,
  downsample = 1000,
  scale = TRUE,
  center = TRUE,
  color_breaks = c(-2, 0, 2),
  colors = c("blue", "white", "red"),
  cell_attributes = NULL,
  col_gap = NULL,
  use_raster = TRUE,
  raster_quality = 10,
  cluster_rows = TRUE,
  cluster_columns = FALSE,
  border = TRUE,
  show_column_names = FALSE,
  ...
)
}
\arguments{
\item{activity_matrix}{A matrix of values, such as TF activities inferred from calculateActivity}

\item{sce}{A SingleCellExperiment object containing information of cell attributes}

\item{tfs}{A character vector indicating the names of the transcription factors to be plotted}

\item{downsample}{Integer indicating the number of cells to sample from the matrix}

\item{scale}{Logical indicating whether to scale the heatmap}

\item{center}{Logical indicating whether to center the heatmap}

\item{color_breaks}{A vector indicating numeric breaks as input to \code{circlize::colorRamp2}}

\item{colors}{A vector of colors corresponding to values in \code{breaks} as input to \code{circlize::colorRamp2}}

\item{cell_attributes}{A character vector matching the column names of \code{colData(sce)} to be used for plotting}

\item{col_gap}{String indicating the cell attribute to split the columns of the heatmap by}

\item{use_raster}{Logical indicating whether to use rasterization to reduce image size}

\item{raster_quality}{Integer indicating the raster quality. The higher the value, the better the resolution}

\item{cluster_rows}{Logical indicating whether to cluster rows}

\item{cluster_columns}{Logical indicating whether to cluster columns}

\item{border}{Logical indicating whether to add border around heatmap}

\item{show_column_names}{Logical indicating whether to show column names}

\item{...}{other arguments for \code{ComplexHeatmap::Heatmap}}
}
\value{
A Heatmap-class object.
}
\description{
Plot transcription factor activity
}
\examples{
example_sce <- scuttle::mockSCE()
example_sce <- scuttle::logNormCounts(example_sce)
example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
activity_matrix <- matrix(rnorm(10*200), nrow=10, ncol=200)
rownames(activity_matrix) <- sample(rownames(example_sce),10)
plotHeatmapActivity(activity_matrix=activity_matrix, sce=example_sce,
tfs=rownames(activity_matrix), cell_attributes='cluster', col_gap='cluster')
}
\author{
Xiaosai Yao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotBubble}
\alias{plotBubble}
\title{Generate bubble plots of relative activities across cluster/group labels for a list of TFs}
\usage{
plotBubble(
  activity_matrix,
  tf,
  clusters,
  bubblesize = c("log.FDR", "summary.logFC", "summary.diff"),
  color.theme = "viridis",
  legend.label = "relative_activity",
  x.label = "clusters",
  y.label = "transcription factors",
  title = "TF activity",
  ...
)
}
\arguments{
\item{activity_matrix}{A matrix of TF activities inferred from calculateActivity}

\item{tf}{A character vector indicating the names of the transcription factors to be plotted}

\item{clusters}{A character or integer vector of cluster or group labels for single cells}

\item{bubblesize}{String indicating the variable from findDifferentialActivity output to scale size of bubbles
by either \code{log.FDR}, \code{summary.logFC} or \code{summary.diff}. Default is \code{logFDR}.}

\item{color.theme}{String indicating the color theme used for the bubble plot and corresponding to the color options
in \code{scale_color_viridis_c}}

\item{legend.label}{String indicating the name of legend corresponding to the color scale}

\item{x.label}{String indicating the x axis label}

\item{y.label}{String indicating the y axis label}

\item{title}{String indicating the title of the plot}

\item{...}{Additional arguments to pass to \code{findDifferentialActivity}}
}
\value{
A ggplot object
}
\description{
Generate bubble plots of relative activities across cluster/group labels for a list of TFs
}
\examples{
example_sce <- scuttle::mockSCE()
example_sce <- scuttle::logNormCounts(example_sce)
example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
plotBubble(activity_matrix = logcounts(example_sce),
tf = c('Gene_0001','Gene_0002'),  clusters = example_sce$cluster)
}
\author{
Shang-yang Chen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{plotGseaNetwork}
\alias{plotGseaNetwork}
\title{Plot networks graph of significant genesets from regulonEnrich results}
\usage{
plotGseaNetwork(
  tf,
  enrichresults,
  ntop_pathways = 10,
  p.adj_cutoff = 0.05,
  layout = "sugiyama",
  tf_label = "tf",
  gset_label = "ID",
  tf_color = "tomato",
  gset_color = "grey"
)
}
\arguments{
\item{tf}{A vector of gene names to be plotted. They should be present in enrichresults}

\item{enrichresults}{Output from regulonEnrich that computes enriched genesets from user-specified regulons
of interest}

\item{ntop_pathways}{An integer indicating the number of top pathways to be included in the graph}

\item{p.adj_cutoff}{A scalar indicating the p.adjusted cutoff for pathways to be included in the graph. Default value is 0.05}

\item{layout}{String indicating layout option from igraph}

\item{tf_label}{String indicating the name of the tf label}

\item{gset_label}{String indicating the name of the geneset label}

\item{tf_color}{String indicating the color of the tf label}

\item{gset_color}{String indicating the color of the geneset label}
}
\value{
an igraph plot of interconnected pathways through TFs
}
\description{
Plot networks graph of significant genesets from regulonEnrich results
}
\examples{
AR <- data.frame(ID = c('ANDROGEN RESPONSE','PROLIFERATION','MAPK'),
p.adjust = c(0.001, 0.01, 0.04))
GATA6 <- data.frame(ID = c('STK33','PROLIFERATION','MAPK'),
p.adjust = c(0.001, 0.01, 0.04))
enrichresults <- list(AR = AR, GATA6 = GATA6)
plotGseaNetwork(tf = names(enrichresults), enrichresults = enrichresults)
}
\author{
Phoebe Guo, Xiaosai Yao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotActivityViolin}
\alias{plotActivityViolin}
\title{Generate violin plots of inferred activities for a list of TFs grouped by cluster/group labels}
\usage{
plotActivityViolin(
  activity_matrix,
  tf,
  clusters,
  ncol = NULL,
  nrow = NULL,
  combine = TRUE,
  legend.label = "activity",
  colors = NULL,
  title = NULL,
  text_size = 10,
  facet_grid_variable = NULL,
  boxplot = FALSE
)
}
\arguments{
\item{activity_matrix}{A matrix of TF activities inferred from calculateActivity}

\item{tf}{A character vector indicating the names of the transcription factors to be plotted}

\item{clusters}{A vector of cluster or group labels for single cells}

\item{ncol}{A integer to indicate the number of columns in the combined plot, if \code{combine = TRUE}}

\item{nrow}{A integer to indicate the number of rows in the combined plot, if \code{combine = TRUE}}

\item{combine}{logical to indicate whether to combine and visualize the plots in one panel}

\item{legend.label}{String indicating the name of variable to be plotted on the legend}

\item{colors}{A character vector representing the names of colors}

\item{title}{String indicating the title of the plot if \code{combine = TRUE}}

\item{text_size}{Scalar indicating the font size of the title}

\item{facet_grid_variable}{A character vector of a secondary label to split the plots by facet_grid}

\item{boxplot}{logical indicating whether to add boxplot on top of violin plot}
}
\value{
A combined ggplot object or a list of ggplots if \code{combine = FALSE}
}
\description{
Generate violin plots of inferred activities for a list of TFs grouped by cluster/group labels
}
\examples{
# create a mock singleCellExperiment object for gene expression matrix
example_sce <- scuttle::mockSCE()
example_sce <- scuttle::logNormCounts(example_sce)
example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
plotActivityViolin(activity_matrix = logcounts(example_sce),
tf = c('Gene_0001','Gene_0002'),  clusters = example_sce$cluster)

}
\author{
Xiaosai Yao, Shang-yang Chen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotActivityDim}
\alias{plotActivityDim}
\title{Plot cell-level reduced dimension results stored in a SingleCellExperiment object, colored by activities for a list of TFs}
\usage{
plotActivityDim(
  sce = NULL,
  activity_matrix,
  tf,
  dimtype = "UMAP",
  label = NULL,
  ncol = NULL,
  nrow = NULL,
  title = NULL,
  combine = TRUE,
  legend.label = "activity",
  colors = c("blue", "yellow"),
  limit = NULL,
  ...
)
}
\arguments{
\item{sce}{A SingleCellExperiment object containing dimensionality reduction coordinates}

\item{activity_matrix}{A matrix of TF activities inferred from calculateActivity}

\item{tf}{A character vector indicating the names of the transcription factors to be plotted}

\item{dimtype}{String indicating the name of dimensionality reduction matrix to be extracted from the SingleCellExperiment}

\item{label}{String corresponding to the field in the colData of sce for annotation on plot}

\item{ncol}{A integer to specify the number of columns in the combined plot, if combine == TRUE}

\item{nrow}{A integer to specify the number of rows in the combined plot, if combine == TRUE}

\item{title}{A string to specify the name of the combined plot}

\item{combine}{logical to indicate whether to combine and visualize the plots in one panel}

\item{legend.label}{String indicating the name of variable to be plotted on the legend}

\item{colors}{A vector of 2 colors for the intensity, with the first element referring to the lower value and
the second element referring to the higher value. Default is c('blue','yellow').}

\item{limit}{A vector of lower and upper bounds for the color scale. The default option is NULL and will adjust
to minimal and maximal values}

\item{...}{Additional arguments from scater::plotReducedDim}
}
\value{
A combined ggplot object or a list of ggplots if combine == FALSE
}
\description{
Plot cell-level reduced dimension results stored in a SingleCellExperiment object, colored by activities for a list of TFs
}
\examples{
# create a mock singleCellExperiment object for gene expression matrix
example_sce <- scuttle::mockSCE()
example_sce <- scuttle::logNormCounts(example_sce)
example_sce <- scater::runPCA(example_sce)
example_sce <- scater::runUMAP(example_sce)
example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
plotActivityDim(sce = example_sce, activity = logcounts(example_sce),
tf = c('Gene_0001','Gene_0002'),  label = 'cluster')
}
\author{
Xiaosai Yao, Shang-yang Chen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{enrichPlot}
\alias{enrichPlot}
\title{Plot results of regulonEnrich}
\usage{
enrichPlot(results, top = 15, ncol = 3, title = NULL, combine = TRUE)
}
\arguments{
\item{results}{Output from regulonEnrich}

\item{top}{An integer to indicate the number of pathways to plot ranked by significance. Default is 15.}

\item{ncol}{An integer to indicate the number of columns in the combined plot, if combine == TRUE. Default is 3.}

\item{title}{String indicating the title of the combined plot}

\item{combine}{logical to indicate whether to combine and visualize the plots in one panel}
}
\value{
A combined ggplot object or a list of ggplots if combine == FALSE
}
\description{
Plot results of regulonEnrich
}
\examples{
#retrieve genesets
msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')

#convert genesets to be compatible with enricher
msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) \%in\% c("c6", "h")}))]
gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
 data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))}))

head(gs.list)

#get regulon
library(dorothea)
data(dorothea_hs, package = 'dorothea')
regulon <- dorothea_hs
enrichment_results <- regulonEnrich(c('ESR1','AR'), regulon = regulon, weight = 'mor',
genesets = gs.list)

# plot graph
enrichPlot(results = enrichment_results )

}
\author{
Xiaosai Yao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulonSimilarities.R
\name{permuteGraph}
\alias{permuteGraph}
\title{Calculate similarity score from permuted graphs to estimate background similarity}
\usage{
permuteGraph(graph, focal_tf, n = 100, p = 1)
}
\arguments{
\item{graph}{an igraph object from \code{buildGraph} or \code{buildDiffGraph}}

\item{focal_tf}{character string indicating the name of the transcription factors to
calculate similarity score}

\item{n}{an integer indicating the number of permutations}

\item{p}{a scalar indicating the probability of rewiring the graphs}
}
\value{
A matrix with Jaccard similarity between the focal transcription factor and all pairs of transcription factors
for n permuted graphs
}
\description{
Calculate similarity score from permuted graphs to estimate background similarity
}
\examples{
regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
target = sample(letters[5:14], 100, replace = TRUE))
regulon$weights <- runif(100)
GRN_graph <- buildGraph(regulon)
permuted_graph <- permuteGraph(GRN_graph, focal_tf = "a")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphs.R
\name{plotDiffNetwork}
\alias{plotDiffNetwork}
\title{Plot graph according to grouping factor}
\usage{
plotDiffNetwork(
  regulon,
  cutoff = 0.01,
  tf = NULL,
  weight = "weight",
  clusters,
  layout = "stress"
)
}
\arguments{
\item{regulon}{an object returned by the getRegulon or addWeights function}

\item{cutoff}{a numeric used to select values of the variables passed in \code{clusters}
parameter. Values greater than \code{cutoff} are retained and used as
graph edge weights.}

\item{tf}{a character vector storing the names of transcription factors to be
included in the graph}

\item{weight}{a string indicating the name of the column in the regulon to be used as
the weight of the edges}

\item{clusters}{a character vector indicating the clusters to be plotted}

\item{layout}{a layout specification. Any values that are valid for
\link[ggraph]{ggraph} or \link[ggraph]{create_layout} will work.}
}
\value{
a ggraph object
}
\description{
Plot graph with separate weights for different levels of the grouping factor
}
\examples{
#' # create an artificial getRegulon output
set.seed(1234)
tf_set <- apply(expand.grid(LETTERS[1:10], LETTERS[1:10]),1,  paste, collapse = '')
regulon <- S4Vectors::DataFrame(tf = sample(tf_set, 5e3, replace = TRUE))
gene_set <- expand.grid(LETTERS[1:10], LETTERS[1:10], LETTERS[1:10])
gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
regulon$target <- sample(gene_set, 5e3, replace = TRUE)
regulon$idxATAC <- 1:5e3
regulon$weight <- cbind(data.frame(C1 = runif(5e3), C2 = runif(5e3),
C3 = runif(5e3)))
plotDiffNetwork(regulon, tf = unique(tf_set)[1:3],
clusters = c('C1', 'C2', 'C3'), cutoff = 0.2)
}
\author{
Xiaosai Yao, Tomasz Wlodarczyk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{regulon}
\alias{regulon}
\title{regulon created using \code{epiregulon} package from reprogram-seq data}
\format{
a DFrame.
}
\usage{
data(regulon)
}
\value{
a DFrame.
}
\description{
regulon created using \code{epiregulon} package from reprogram-seq data
}
\examples{
data(regulon)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulonSimilarities.R
\name{findPartners}
\alias{findPartners}
\title{Find interaction partners of a transcription factor of interest}
\usage{
findPartners(graph, focal_tf)
}
\arguments{
\item{graph}{a igraph object from \code{buildGraph} or \code{buildDiffGraph}}

\item{focal_tf}{character string indicating the name of the transcription factors to find
interaction partners of}
}
\value{
A list with elements corresponding to each transcription factor apart from
the focal one. Each list element is represented as a data frame with columns containing
names of all target genes shared with focal transcription factor, weights of edges
connecting transcription factor with target genes, equivalent weights for focal transcription
factor and the element wise product of both weight columns.
}
\description{
Find interaction partners of a transcription factor of interest
}
\examples{
regulon <- data.frame(tf = sample(letters[1:4], 100, replace = TRUE), idxATAC= 1:100,
target = sample(letters[5:14], 100, replace = TRUE))
regulon$weights <- runif(100)
GRN_graph <- buildGraph(regulon)
partners <- findPartners(GRN_graph, 'a')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotHeatmapRegulon}
\alias{plotHeatmapRegulon}
\title{Plot targets genes of transcription factors in regulons}
\usage{
plotHeatmapRegulon(
  sce,
  tfs,
  regulon,
  regulon_column = "weight",
  regulon_cutoff = 0.1,
  downsample = 1000,
  scale = TRUE,
  center = TRUE,
  color_breaks = c(-2, 0, 2),
  colors = c("blue", "white", "red"),
  cell_attributes,
  col_gap = NULL,
  exprs_values = "logcounts",
  use_raster = TRUE,
  raster_quality = 10,
  cluster_rows = FALSE,
  cluster_columns = FALSE,
  border = TRUE,
  show_column_names = FALSE,
  column_col = NULL,
  row_col = NULL,
  genes_label = NULL,
  ...
)
}
\arguments{
\item{sce}{A SingleCellExperiment object containing information of cell attributes}

\item{tfs}{A character vector indicating the names of the transcription factors to be plotted}

\item{regulon}{A dataframe of regulons containing \code{tf}, \code{targets} and a column for filtering the regulons}

\item{regulon_column}{String indicating the column names to be used for filtering regulons}

\item{regulon_cutoff}{A scalar indicating the minimal value to retain the regulons for plotting}

\item{downsample}{Integer indicating the number of cells to sample from the matrix}

\item{scale}{Logical indicating whether to scale the heatmap}

\item{center}{Logical indicating whether to center the heatmap}

\item{color_breaks}{A vector indicating numeric breaks as input to \code{circlize::colorRamp2}}

\item{colors}{A vector of colors corresponding to values in \code{breaks} as input to \code{circlize::colorRamp2}}

\item{cell_attributes}{A character vector matching the column names of \code{colData(sce)} to be used for plotting}

\item{col_gap}{String indicating the cell attribute to split the columns of the heatmap by}

\item{exprs_values}{A string specifying which assay in \code{assays(object)} to obtain expression values from}

\item{use_raster}{Logical indicating whether to use rasterization to reduce image size}

\item{raster_quality}{Integer indicating the raster quality. The higher the value, the better the resolution}

\item{cluster_rows}{Logical indicating whether to cluster rows}

\item{cluster_columns}{Logical indicating whether to cluster columns}

\item{border}{Logical indicating whether to add border around heatmap}

\item{show_column_names}{Logical indicating whether to show column names}

\item{column_col}{A list specifying the colors in the columns.
See \href{https://jokergoo.github.io/ComplexHeatmap-reference/book/heatmap-annotations.html#simple-annotation}{here}}

\item{row_col}{A list specifying the colors in the rows.
See \href{https://jokergoo.github.io/ComplexHeatmap-reference/book/heatmap-annotations.html#simple-annotation}{here}}

\item{genes_label}{A character vector indicating a selected list of genes to show on the rownames}

\item{...}{other arguments for \code{ComplexHeatmap::Heatmap}}
}
\value{
A Heatmap-class object.
}
\description{
Plot targets genes of transcription factors in regulons
}
\examples{
example_sce <- scuttle::mockSCE()
example_sce <- scuttle::logNormCounts(example_sce)
example_sce$cluster <- sample(LETTERS[1:5], ncol(example_sce), replace = TRUE)
regulon <- data.frame(tf=c(rep('Gene_0001',10),rep('Gene_0002',20)),
target = sample(rownames(example_sce),30), weight = rnorm(30))
#plot heatmap and rotate labels
plotHeatmapRegulon(example_sce, tfs=c('Gene_0001','Gene_0002'), regulon=regulon,
cell_attributes='cluster', col_gap = 'cluster', column_title_rot = 90)
}
\author{
Xiaosai Yao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{regulonEnrich}
\alias{regulonEnrich}
\title{Perform geneset enrichment of user-defined regulons}
\usage{
regulonEnrich(TF, regulon, weight = "weight", weight_cutoff = 0.5, genesets)
}
\arguments{
\item{TF}{A character vector of TF names}

\item{regulon}{A matrix of weighted regulon consisting of tf, targets, corr and weight}

\item{weight}{String indicating the column name that should be used to filter target genes for geneset enrichment. Default is 'weight'.}

\item{weight_cutoff}{A numeric scalar to indicate the cutoff to filter on the column specified by \code{weight}. Default is 0.5.}

\item{genesets}{A dataframe with the first column being the name of the geneset and the second column being the name of the genes}
}
\value{
A dataframe showing the significantly enriched pathways
}
\description{
Perform geneset enrichment of user-defined regulons
}
\examples{
#retrieve genesets
msigdb.hs = msigdb::getMsigdb(org = 'hs', id = 'SYM', version = '7.4')

#convert genesets to be compatible with enricher
msigdb.hs <- msigdb.hs[unlist(lapply(msigdb.hs, function(x) {GSEABase::bcCategory(GSEABase::collectionType(x)) \%in\% c("c6", "h")}))]
gs.list <- do.call(rbind, lapply(names(msigdb.hs), function(x) {
 data.frame(gs = x, genes = GSEABase::geneIds(msigdb.hs[x][[1]]))}))

head(gs.list)

#get regulon
library(dorothea)
data(dorothea_hs, package = 'dorothea')
regulon <- dorothea_hs
enrichment_results <- regulonEnrich(c('ESR1','AR'), regulon = regulon, weight = 'mor',
genesets = gs.list)

}
\author{
Xiaosai Yao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphs.R
\name{buildGraph}
\alias{buildGraph}
\alias{buildDiffGraph}
\alias{addCentrality}
\alias{normalizeCentrality}
\alias{rankTfs}
\title{Creating graphs and related operations}
\usage{
buildGraph(
  regulon,
  mode = c("tg", "tripartite", "re", "pairs"),
  weights = "weights",
  cluster = "all",
  aggregation_function = function(x) x[which.max(abs(x))],
  na_replace = TRUE,
  keep_original_names = TRUE,
  filter_edges = NULL
)

buildDiffGraph(graph_obj_1, graph_obj_2, weighted = TRUE, abs_diff = TRUE)

addCentrality(graph)

normalizeCentrality(graph, FUN = sqrt, weighted = TRUE)

rankTfs(graph, type_attr = "type")
}
\arguments{
\item{regulon}{an object returned by the getRegulon or addWeights function.}

\item{mode}{a character specifying which type of graph will be built. In \code{'tg'} mode
a bipartite graph is built by connecting transcription factors directly to the target genes
and ignoring information about mediating regulatory elements; in \code{'pairs'} mode
transcription factors are connected to unique target gene-regulatory element pairs;
in \code{'tripartite'} mode the network is made up of three types of vertices (nodes):
transcription factors, regulatory elements and target genes; here the path from
target gene to regulatory element always contains a regulatory element; in
\code{'re'} mode data in the target genes is dropped and only connections are
between transcription factors and regulatory elements.}

\item{weights}{a character specifying which variable should be used to assign
weights to edges.}

\item{cluster}{a character specifying the name of the cluster column which should be used
to retrieve weight values from regulon object. Using this argument makes sense
only with combination with \code{weights} parameter when it points to the regulon column
that is a matrix.}

\item{aggregation_function}{a function used to aggregate weights of duplicated edges,
which might appear due to the many transcription factor converging at the same regulatory
element; starting from this point each transcription factor is supposed to have a separate
connection to the target gene, perhaps the same one across several connections. In
\code{tripartite} mode this might result in many edges in the same node pair, however weights might
differ since they are inherited from different tf-re-tg triplets (rows) in the
\code{regulon} object. Similarly, duplicated edges are generated by one
transcription factor using a regulatory element multiple times to reach different
target genes. In \code{tg} mode the edges became duplicated if one transcription
factor reaches the same target genes through many regulatory elements.}

\item{na_replace}{a logical indicating whether NA values for weights should be
replaced with zeros.}

\item{keep_original_names}{A logical indicating whether gene names should be used as
node names in the output graph. Note that this might lead to the duplicated node
names if the same gene is present in two layers (transcription factors and target genes).}

\item{filter_edges}{A numeric defining the cutoff weight used for filtering out edges
which have weights equal or greater than cutoff. The isolated vertices are removed then
from the graph. Defaults to NULL in which case no filtering is applied.}

\item{weighted}{a logical indicating whether weighted graphs are used; in \code{tripartite}
mode tf-re-tg triplet is decomposed into two edges corresponding to tf-re and re-tg pairs,
and both edges inherit the same weight, which was originally assigned to the parent triplet.}

\item{abs_diff}{a logical indicating whether absolute difference in the number
edges or their weights will be calculated.}

\item{graph, graph_obj_1, graph_obj_2}{an igraph object.}

\item{FUN}{a function used for normalization. The input to this
function is be the number of edges connected with each node (incident edges).}

\item{type_attr}{a character corresponding to the name of the vertex attribute
which indicate the type of vertex.}
}
\value{
an igraph object.
\code{rankTfs} returns a data.frame with transcription factors sorted according
to the value of the \code{centrality} attribute.
}
\description{
The function enable to create graph objects using as input regulon objects returned by
\code{pruneRegulon} or \code{addWeights}. Both weighted and unweighted graphs can be
created that can further be visualized using dedicated functions.
}
\details{
\code{buildGraph} function creates a directed graph based on the output of
the \code{getRegulon} function. Four modes are available: (1) \code{tg} in which
connections are made directly between transcription factor and target genes. Even if
the same tf-tg pair is connected in the original regulon object through many
regulatory elements then only one edge is created. In such a case, when weighted
graph is created, weights are summarized by the aggregating function (by default
the maximum absolute value with the sign of the original value). Similarly, aggregation
is made in the \code{re} mode leaving only unique transcription factor-regulatory element pairs.
In \code{tripartite} mode edges connect transcription factors with regulatory elements and
regulatory elements with target genes. The same weights are used for both edges
that correspond to the single row in the regulon data frame (tf-re and re-tg). Note
that the original regulon structure is not fully preserved because each row is now
represented by two edges which are independent from each other. Thus they can be
coupled with different edges connected to the same regulatory element building the
path from transcription factor to the target gene of another transcription factor
through the shared regulatory element.

\code{buildDiffGraph} a graph difference by subtracting the edges of \code{graph_obj_2}
from those of the \code{graph_obj_1}. If \code{weighted} is set to \code{TRUE} then for each
ordered pair of vertices (nodes) the difference in number of edges between \code{graph_obj_1}
and \code{graph_obj_1} is calculated. The result is used to set the number of
corresponding edges in output graph. Note that unless \code{abs_diff} is set to
\code{TRUE} any non-positive difference will translate into lack of the edges
for a corresponding ordered pair of vertices in the output graph (equivalent
to 0 value in the respective position in adjacency matrix). In case of
weighted graphs, the weight of the output graph is calculated as a difference
of the corresponding weights between input graphs.

\code{addCentrality} calculates degree centrality for each vertex using
\code{igraph::strength}.

With \code{normalizeCentrality} function the normalized values of centrality
are calculated from the original ones divided by
\code{FUN}(total number of non-zero edges associated with each node).

\code{rankTfs} assign ranks to transcription factors according to degree
centrality of their vertices
}
\examples{
# create an artificial getRegulon output
set.seed(1234)
tf_set <- apply(expand.grid(LETTERS[1:10], LETTERS[1:10]),1,  paste, collapse = '')
regulon <- DataFrame(tf = sample(tf_set, 5e3, replace = TRUE))
gene_set <- expand.grid(LETTERS[1:10], LETTERS[1:10], LETTERS[1:10])
gene_set <- apply(gene_set,1,function(x) paste0(x,collapse=''))
regulon$target <- sample(gene_set, 5e3, replace = TRUE)
regulon$idxATAC <- 1:5e3
regulon$corr <- runif(5e3)*0.5+0.5
regulon$weights <- matrix(runif(15000), nrow=5000, ncol=3)
colnames(regulon$weights) <- c('all','cluster1', 'cluster2')
graph_tripartite <- buildGraph(regulon, cluster='all', mode = 'tripartite')

# build bipartite graph using regulatory element-target gene pairs
graph_pairs_1 <- buildGraph(regulon, cluster = 'cluster1', mode = 'pairs')
graph_pairs_2 <- buildGraph(regulon, cluster = 'cluster2', mode = 'pairs')
graph_diff <- buildDiffGraph(graph_pairs_1, graph_pairs_2)
graph_diff <- addCentrality(graph_diff)
graph_diff <- normalizeCentrality(graph_diff)
tf_ranking <- rankTfs(graph_diff)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{findDifferentialActivity}
\alias{findDifferentialActivity}
\title{Test for differential TF activity between pairs of single cell clusters/groups}
\usage{
findDifferentialActivity(
  activity_matrix,
  clusters,
  test.type = c("t", "wilcox", "binom"),
  pval.type = c("some", "any", "all"),
  direction = c("any", "up", "down"),
  logvalues = TRUE,
  ...
)
}
\arguments{
\item{activity_matrix}{A matrix of TF activities inferred from calculateActivity}

\item{clusters}{A character or integer vector of cluster or group labels for single cells}

\item{test.type}{String indicating the type of statistical tests to be passed to scran::findMarkers, can be "t", "wilcox". or "binom"}

\item{pval.type}{A string specifying how p-values are to be combined across pairwise comparisons for a given group/cluster. For more details see \link[scran]{combineMarkers}.}

\item{direction}{A string specifying direction of differential TF activity, can be "any", "up" or "down"}

\item{logvalues}{logical indicating whether activities are computed from logged gene expression or not.
If activity is computed from linear values of gene expression, setting logvalues to FALSE will return the difference.
If activity is computed from logged values of gene expression, setting logvalues to TRUE will return the log changes.}

\item{...}{Further arguments to pass to scran::findMarkers}
}
\value{
A named list of dataframes containing differential TF activity test results for each cluster/group
}
\description{
Test for differential TF activity between pairs of single cell clusters/groups
}
\examples{
set.seed(1)
score.combine <- cbind(matrix(runif(2000,0,2), 20,100), matrix(runif(2000,0,10), 20,100))
rownames(score.combine) <- paste0("TF",1:20)
colnames(score.combine) <- paste0("cell",1:200)
cluster <- c(rep(1,100),rep(2,100))
markers <- findDifferentialActivity(
activity_matrix = score.combine,
clusters = cluster,
pval.type = "some",
direction = "up",
test.type = "t")
sig.genes <- getSigGenes(markers, fdr_cutoff = 1, summary_cutoff = 0.1)
}
\author{
Xiaosai Yao, Shang-yang Chen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statisticalTests.R
\name{getSigGenes}
\alias{getSigGenes}
\title{Compile and summarize the output from findDifferentialActivity function}
\usage{
getSigGenes(
  da_list,
  fdr_cutoff = 0.05,
  summary_cutoff = NULL,
  topgenes = NULL,
  direction = c("any", "up", "down")
)
}
\arguments{
\item{da_list}{List of dataframes from running findDifferentialActivity}

\item{fdr_cutoff}{A numeric scalar to specify the cutoff for FDR value. Default is 0.05}

\item{summary_cutoff}{A numeric scalar to specify the cutoff for log fold change or difference}

\item{topgenes}{A integer scalar to indicate the number of top ordered genes to include in output}

\item{direction}{A string specifying direction for which differential TF activity was calculated, can be "any", "up" or "down"}
}
\value{
A compiled dataframe of TFs with differential activities across clusters/groups
}
\description{
Compile and summarize the output from findDifferentialActivity function
}
\examples{
set.seed(1)
score.combine <- cbind(matrix(runif(2000,0,2), 20,100), matrix(runif(2000,0,10), 20,100))
rownames(score.combine) <- paste0("TF",1:20)
colnames(score.combine) <- paste0("cell",1:200)
cluster <- c(rep(1,100),rep(2,100))
markers <- findDifferentialActivity(score.combine, cluster, pval.type = "some", direction = "up",
test.type = "t")
sig.genes <- getSigGenes(markers, fdr_cutoff = 1, summary_cutoff = 0.1)
utils::head(sig.genes)
}
\author{
Xiaosai Yao, Shang-yang Chen
}
